@@ -2,8 +2,9 @@ import lamindb as ln
 
 ln.examples.datasets.mini_immuno.define_features_labels()
 adata = ln.examples.datasets.mini_immuno.get_dataset1(otype="AnnData")
-schema = ln.examples.schemas.anndata_ensembl_gene_ids_and_valid_features_in_obs()
 artifact = ln.Artifact.from_anndata(
-    adata, key="examples/mini_immuno.h5ad", schema=schema
+    adata,
+    key="examples/mini_immuno.h5ad",
+    schema="ensembl_gene_ids_and_valid_features_in_obs",
 ).save()
 artifact.describe()
