@@ -13,7 +13,7 @@ from typing import TYPE_CHECKING, TextIO
 
 import lamindb_setup as ln_setup
 from django.db.models import Func, IntegerField, Q
-from lamin_utils import logger
+from lamin_utils._logger import logger, set_handler
 from lamindb_setup.core import deprecated
 from lamindb_setup.core.hashing import hash_file
 
@@ -88,6 +88,18 @@ def get_notebook_key_colab() -> str:
     return key
 
 
+def get_cli_args() -> str | None:
+    """Returns the CLI arguments used to invoke a script.
+
+    Returns None if not run as a script (e.g., in Jupyter, interactive shell).
+    """
+    # check if run as a script (not interactive/notebook/imported)
+    # and whether arguments have been passed
+    if len(sys.argv) > 1 and sys.argv[0] and not is_run_from_ipython:
+        return " ".join(sys.argv[1:])
+    return None
+
+
 def pretty_pypackages(dependencies: dict) -> str:
     deps_list = []
     for pkg, ver in dependencies.items():
@@ -167,6 +179,10 @@ class LogStreamTracker:
             ln_setup.settings.cache_dir / f"run_logs_{self.run.uid}.txt"
         )
         self.log_file = open(self.log_file_path, "w")
+        # the instance that's connected is important information
+        self.log_file.write(
+            f"\x1b[92m→\x1b[0m connected lamindb: {ln_setup.settings.instance.slug}\n"
+        )
         # use buffering for correct handling of carriage returns
         sys.stdout = LogStreamHandler(
             self.original_stdout, self.log_file, use_buffer=True
@@ -183,6 +199,8 @@ class LogStreamTracker:
             signal.signal(signal.SIGINT, self.cleanup)
         # handle exceptions
         sys.excepthook = self.handle_exception
+        # reset handler for lamin logger because sys.stdout has been replaced
+        set_handler(logger)
 
     def finish(self):
         if self.original_stdout:
@@ -191,6 +209,8 @@ class LogStreamTracker:
             sys.stdout = self.original_stdout
             sys.stderr = self.original_stderr
             self.log_file.close()
+            # reset handler for lamin logger because sys.stdout has been replaced
+            set_handler(logger)
 
     def cleanup(self, signo=None, frame=None):
         try:
@@ -236,19 +256,29 @@ class LogStreamTracker:
         self.original_excepthook(exc_type, exc_value, exc_traceback)
 
 
+# see test_tracked.py for tests
 def serialize_params_to_json(params: dict) -> dict:
     serialized_params = {}
     for key, value in params.items():
-        if isinstance(value, SQLRecord):
-            value = f"{value.__class__.__get_name_with_module__()}[{value.uid}]"
-        else:
-            dtype, _, _ = infer_feature_type_convert_json(key, value)
-            if (dtype == "?" or dtype.startswith("cat")) and dtype != "cat ? str":
-                logger.warning(
-                    f"skipping param {key} because dtype not JSON serializable"
+        dtype, converted_value, _ = infer_feature_type_convert_json(key, value)
+        if (
+            dtype == "?" or dtype.startswith("cat") or dtype.startswith("list[cat")
+        ) and dtype != "cat ? str":
+            if isinstance(value, SQLRecord):
+                serialized_params[key] = (
+                    f"{value.__class__.__get_name_with_module__()}[{value.uid}]"
                 )
-                continue
-        serialized_params[key] = value
+            elif dtype.startswith("list[cat"):
+                items = list(value)
+                if items and all(isinstance(item, SQLRecord) for item in items):
+                    serialized_params[key] = [  # type: ignore
+                        f"{item.__class__.__get_name_with_module__()}[{item.uid}]"
+                        for item in items
+                    ]
+        else:
+            serialized_params[key] = converted_value
+        if key not in serialized_params:
+            logger.warning(f"skipping param {key} because dtype not JSON serializable")
     return serialized_params
 
 
@@ -529,13 +559,17 @@ class Context:
         if params is not None:
             run.params = serialize_params_to_json(params)
             self._logging_message_track += "\n→ params: " + ", ".join(
-                f"{key}={value}" for key, value in run.params.items()
+                f"{key}={value!r}" for key, value in run.params.items()
             )
+        cli_args = get_cli_args()
+        if cli_args:
+            logger.important(f"script invoked with: {cli_args}")
+            run.cli_args = cli_args
         run.save()  # need to save now
         if features is not None:
             run.features.add_values(features)
             self._logging_message_track += "\n→ features: " + ", ".join(
-                f"{key}={value}" for key, value in features.items()
+                f"{key}={value!r}" for key, value in features.items()
             )
         self._run = run
         track_python_environment(run)
@@ -577,7 +611,12 @@ class Context:
                 f'recommendation: to identify the {notebook_or_script} across renames, pass the uid: ln{r_or_python}track("{self.transform.uid[:-4]}"{kwargs_str})'
             )
         if self.transform.type == "script":
-            save_context_core(run=run, transform=self.transform, filepath=self._path)
+            save_context_core(
+                run=run,
+                transform=self.transform,
+                filepath=self._path,
+                message_prefix="monitor at",
+            )
 
     def _track_source_code(
         self,
@@ -738,8 +777,8 @@ class Context:
             aux_transform = None
 
         # determine the transform key
-        if ln_setup.settings.work_dir is not None:
-            key = self._path.relative_to(ln_setup.settings.work_dir).as_posix()
+        if ln_setup.settings.dev_dir is not None:
+            key = self._path.relative_to(ln_setup.settings.dev_dir).as_posix()
         else:
             key = self._path.name
         # if the user did not pass a uid and there is no matching aux_transform
@@ -886,6 +925,13 @@ class Context:
                 raise UpdateContext(
                     f'{transform.created_by.name} ({transform.created_by.handle}) already works on this draft {transform.type}.\n\nPlease create a revision via `ln.track("{uid[:-4]}{increment_base62(uid[-4:])}")` or a new transform with a *different* key and `ln.track("{base62_12()}0000")`.'
                 )
+            if transform.reference != transform_ref:
+                transform.reference = transform_ref
+                transform.reference_type = transform_ref_type
+                transform.save()
+                self._logging_message_track += (
+                    "updated transform reference, "  # white space on purpose
+                )
             # check whether transform source code was already saved
             if transform_was_saved:
                 bump_revision = False
@@ -974,8 +1020,13 @@ class Context:
                 return None
         else:
             self.run.finished_at = datetime.now(timezone.utc)
+            if ln_setup.settings.instance.is_on_hub:
+                instance_slug = ln_setup.settings.instance.slug
+                ui_url = ln_setup.settings.instance.ui_url
+                logger.important(
+                    f"go to: {ui_url}/{instance_slug}/transform/{self.transform.uid}"
+                )
             save_run_logs(self.run, save_run=True)
-        if self.transform.type != "notebook":
             self._stream_tracker.finish()
         # reset the context attributes so that somebody who runs `track()` after finish
         # starts fresh
