@@ -19,7 +19,7 @@ def get_git_repo_from_remote(url: str | None = None, depth: int | None = 10) ->
     repo_url = url or settings.sync_git_repo
     repo_dir = setup_settings.cache_dir / repo_url.split("/")[-1]
     if repo_dir.exists():
-        logger.warning(f"git repo {repo_dir} already exists locally")
+        logger.debug(f"git repo {repo_dir} already exists locally")
         return repo_dir
     logger.important(
         f"running outside of synched git repo, cloning {repo_url} into {repo_dir}"
@@ -182,8 +182,8 @@ def get_transform_reference_from_git_repo(path: Path) -> str:
         if repo_dir is None:
             repo_dir = Path.cwd()
         raise BlobHashNotFound(
-            f"❌ Did not find blob hash {blob_hash} in git repo ({settings.sync_git_repo}) {repo_dir}\n"
-            f"Did you commit the script? -> {path}"
+            f"❌ Did not find blob hash {blob_hash} in git repo: {settings.sync_git_repo}\n"
+            f"Did you commit & push the script to the remote repo? -> {path}"
         )
     gitpath = get_filepath_within_git_repo(commit_hash, blob_hash, repo_dir)
     reference = f"{settings.sync_git_repo}/blob/{commit_hash}/{gitpath}"
