@@ -1,14 +1,11 @@
 """Curator utilities.
 
-.. autosummary::
-   :toctree: .
-
-   Curator
-   SlotsCurator
-   ComponentCurator
-   CatVector
-   CatLookup
-   DataFrameCatManager
+.. autoclass:: Curator
+.. autoclass:: SlotsCurator
+.. autoclass:: ComponentCurator
+.. autoclass:: CatVector
+.. autoclass:: CatLookup
+.. autoclass:: DataFrameCatManager
 
 """
 
@@ -182,41 +179,50 @@ class Curator:
         - :class:`~lamindb.curators.TiledbsomaExperimentCurator`
     """
 
-    def __init__(self, dataset: Any, schema: Schema | None = None):
+    def __init__(
+        self, dataset: Any, schema: Schema, *, features: dict[str, Any] | None = None
+    ) -> None:
         if not isinstance(schema, Schema):
             raise InvalidArgument("schema argument must be a Schema record.")
-
         if schema.pk is None:
             raise ValueError(
                 "Schema must be saved before curation. Please save it using '.save()'."
             )
-
-        self._artifact: Artifact = None  # pass the dataset as an artifact
-        self._dataset: Any = dataset  # pass the dataset as a UPathStr or data object
-        if isinstance(self._dataset, Artifact):
-            self._artifact = self._dataset
+        self._artifact: Artifact | None = None
+        self._dataset: Any = None
+        # self._dataset is set below, it is opened or loaded if dataset is an Artifact
+        if isinstance(dataset, Artifact):
+            self._artifact = dataset
             if self._artifact.otype in {
                 "DataFrame",
                 "AnnData",
                 "MuData",
                 "SpatialData",
             }:
-                # Open remote AnnData Artifacts
-                if not isinstance(self._artifact.path, LocalPathClasses):
-                    if self._artifact.otype in {
-                        "AnnData",
-                    }:
-                        try:
-                            self._dataset = self._dataset.open(mode="r")
-                        # open can raise various errors. Fall back to loading into memory if open fails
-                        except Exception as e:
-                            logger.warning(
-                                f"Unable to open remote AnnData Artifact: {e}. Falling back to loading into memory."
-                            )
-                            self._dataset = self._dataset.load(is_run_input=False)
-                else:
-                    self._dataset = self._dataset.load(is_run_input=False)
-        self._schema: Schema | None = schema
+                if (
+                    not isinstance(self._artifact.path, LocalPathClasses)
+                    and self._artifact.otype == "AnnData"
+                ):
+                    try:
+                        self._dataset = self._artifact.open(mode="r")
+                        logger.important(
+                            "opened remote artifact for streaming during validation"
+                        )
+                    except Exception as e:
+                        logger.warning(
+                            f"unable to open remote AnnData Artifact: {e}, falling back to loading into memory"
+                        )
+                if self._dataset is None:
+                    logger.important("loading artifact into memory for validation")
+                    self._dataset = self._artifact.load(is_run_input=False)
+            else:
+                raise InvalidArgument(
+                    f"Cannot load or open artifact of this type: {self._artifact}"
+                )
+        else:
+            self._dataset = dataset
+        self._schema: Schema = schema
+        self._external_features: dict[str, Any] = features
         self._is_validated: bool = False
 
     @doc_args(VALIDATE_DOCSTRING)
@@ -311,8 +317,10 @@ class SlotsCurator(Curator):
         self,
         dataset: Artifact | ScverseDataStructures | SOMAExperiment,
         schema: Schema,
+        *,
+        features: dict[str, Any] | None = None,
     ) -> None:
-        super().__init__(dataset=dataset, schema=schema)
+        super().__init__(dataset=dataset, schema=schema, features=features)
         self._slots: dict[str, ComponentCurator] = {}
 
         # used for multimodal data structures (not AnnData)
@@ -330,6 +338,23 @@ class SlotsCurator(Curator):
     @doc_args(VALIDATE_DOCSTRING)
     def validate(self) -> None:
         """{}"""  # noqa: D415
+        if "__external__" in self._schema.slots:
+            validation_schema = self._schema.slots["__external__"]
+            if not self._external_features:
+                if self._artifact is not None and not self._artifact._state.adding:
+                    logger.important(
+                        "no new external features provided, using existing external features of artifact for validation"
+                    )
+                    self._external_features = self._artifact.features.get_values(
+                        external_only=True
+                    )
+                else:
+                    raise ValidationError(
+                        "External features slot is defined in schema but no external features were provided."
+                    )
+            ExperimentalDictCurator(
+                self._external_features, validation_schema
+            ).validate()
         for slot, curator in self._slots.items():
             logger.debug(f"validating slot {slot} ...")
             curator.validate()
@@ -371,7 +396,6 @@ class SlotsCurator(Curator):
                 ),
                 (data_is_soma_experiment, Artifact.from_tiledbsoma),
             ]
-
             for type_check, af_constructor in type_mapping:
                 if type_check(self._dataset):
                     self._artifact = af_constructor(  # type: ignore
@@ -382,13 +406,13 @@ class SlotsCurator(Curator):
                         run=run,
                     )
                     break
-
         cat_vectors = {}
         for curator in self._slots.values():
             for key, cat_vector in curator.cat._cat_vectors.items():
                 cat_vectors[key] = cat_vector
-
         self._artifact.schema = self._schema
+        if self._external_features:
+            self._artifact._external_features = self._external_features
         self._artifact.save()
         return annotate_artifact(  # type: ignore
             self._artifact,
@@ -401,14 +425,13 @@ def convert_dict_to_dataframe_for_validation(d: dict, schema: Schema) -> pd.Data
     """Convert a dictionary to a DataFrame for validation against a schema."""
     df = pd.DataFrame([d])
     for feature in schema.members:
+        # we cannot cast a `list[cat[...]]]` to categorical because lists are not hashable
         if feature.dtype.startswith("cat"):
             if feature.name in df.columns:
                 df[feature.name] = pd.Categorical(df[feature.name])
     return df
 
 
-# This is also currently used as DictCurator by flattening dictionaries into wide DataFrames.
-# Such an approach was never intended and there is room for a DictCurator in the future.
 # For more context, read https://laminlabs.slack.com/archives/C07DB677JF6/p1753994077716099 and
 # https://www.notion.so/laminlabs/Add-a-DictCurator-2422aeaa55e180b9a513f91d13970836
 class ComponentCurator(Curator):
@@ -461,6 +484,7 @@ class ComponentCurator(Curator):
             assert schema.itype is not None  # noqa: S101
 
         pandera_columns = {}
+        self._pandera_schema = None
         if features or schema._index_feature_uid is not None:
             # populate features
             if schema.minimal_set:
@@ -471,7 +495,19 @@ class ComponentCurator(Curator):
                 else:
                     required = False
                 # series.dtype is "object" if the column has lists types, e.g. [["a", "b"], ["a"], ["b"]]
-                if feature.dtype in {
+                if feature.dtype.startswith("list[cat"):
+                    pandera_columns[feature.name] = pandera.Column(
+                        dtype=None,
+                        checks=pandera.Check(
+                            check_dtype("list", feature.nullable),
+                            element_wise=False,
+                            error=f"Column '{feature.name}' failed dtype check for '{feature.dtype}'",
+                        ),
+                        nullable=feature.nullable,
+                        coerce=feature.coerce_dtype,
+                        required=required,
+                    )
+                elif feature.dtype in {
                     "int",
                     "float",
                     "num",
@@ -488,7 +524,7 @@ class ComponentCurator(Curator):
                     pandera_columns[feature.name] = pandera.Column(
                         dtype=None,
                         checks=pandera.Check(
-                            check_dtype(feature.dtype),
+                            check_dtype(feature.dtype, feature.nullable),
                             element_wise=False,
                             error=f"Column '{feature.name}' failed dtype check for '{feature.dtype}': got {dtype}",
                         ),
@@ -536,13 +572,15 @@ class ComponentCurator(Curator):
                 ordered=schema.ordered_set,
                 index=index,
             )
-        # in the DataFrameCatManager, we use the
-        # actual columns of the dataset, not the pandera columns
-        # the pandera columns might have additional optional columns
-        if schema.itype == "Composite":
+        if (
+            schema.itype == "Composite"
+        ):  # backward compat, should be migrated to Feature.name
             columns_field = Feature.name
         else:
             columns_field = parse_cat_dtype(schema.itype, is_itype=True)["field"]
+        # in the DataFrameCatManager, we use the
+        # actual columns of the dataset, not the pandera columns
+        # the pandera columns might have additional optional columns
         self._cat_manager = DataFrameCatManager(
             self._dataset,
             columns_field=columns_field,
@@ -618,7 +656,7 @@ class ComponentCurator(Curator):
     @doc_args(VALIDATE_DOCSTRING)
     def validate(self) -> None:
         """{}"""  # noqa: D415
-        if self._schema.n > 0:
+        if self._pandera_schema is not None:
             try:
                 # first validate through pandera
                 self._pandera_schema.validate(self._dataset, lazy=True)
@@ -634,35 +672,6 @@ class ComponentCurator(Curator):
         else:
             self._cat_manager_validate()
 
-    @doc_args(SAVE_ARTIFACT_DOCSTRING)
-    def save_artifact(
-        self,
-        *,
-        key: str | None = None,
-        description: str | None = None,
-        revises: Artifact | None = None,
-        run: Run | None = None,
-    ) -> Artifact:
-        """{}"""  # noqa: D415
-        if not self._is_validated:
-            self.validate()  # raises ValidationError if doesn't validate
-        if self._artifact is None:
-            self._artifact = Artifact.from_dataframe(
-                self._dataset,
-                key=key,
-                description=description,
-                revises=revises,
-                run=run,
-                format=".csv" if key is not None and key.endswith(".csv") else None,
-            )
-
-        self._artifact.schema = self._schema
-        self._artifact.save()
-        return annotate_artifact(  # type: ignore
-            self._artifact,
-            cat_vectors=self.cat._cat_vectors,
-        )
-
 
 class DataFrameCurator(SlotsCurator):
     # the example in the docstring is tested in test_curators_quickstart_example
@@ -702,16 +711,18 @@ class DataFrameCurator(SlotsCurator):
         self,
         dataset: pd.DataFrame | Artifact,
         schema: Schema,
+        *,
         slot: str | None = None,
+        features: dict[str, Any] | None = None,
     ) -> None:
-        super().__init__(dataset=dataset, schema=schema)
-
+        # loads or opens dataset, dataset may be an artifact
+        super().__init__(dataset=dataset, schema=schema, features=features)
+        # uses open dataset at self._dataset
         self._atomic_curator = ComponentCurator(
-            dataset=dataset,
+            dataset=self._dataset,
             schema=schema,
             slot=slot,
         )
-
         # Handle (nested) attrs
         if slot is None and schema.slots:
             for slot_name, slot_schema in schema.slots.items():
@@ -738,9 +749,7 @@ class DataFrameCurator(SlotsCurator):
     @property
     def cat(self) -> DataFrameCatManager:
         """Manage categoricals by updating registries."""
-        if hasattr(self, "_atomic_curator"):
-            return self._atomic_curator.cat
-        raise AttributeError("cat is only available for slots DataFrameCurator")
+        return self._atomic_curator.cat
 
     def standardize(self) -> None:
         """Standardize the dataset.
@@ -748,20 +757,16 @@ class DataFrameCurator(SlotsCurator):
         - Adds missing columns for features
         - Fills missing values for features with default values
         """
-        if hasattr(self, "_atomic_curator"):
-            self._atomic_curator.standardize()
-        else:
-            for slot_curator in self._slots.values():
-                slot_curator.standardize()
+        self._atomic_curator.standardize()
+        for slot_curator in self._slots.values():
+            slot_curator.standardize()
 
     @doc_args(VALIDATE_DOCSTRING)
     def validate(self) -> None:
         """{}."""
-        if hasattr(self, "_atomic_curator"):
-            self._atomic_curator.validate()
-            self._is_validated = self._atomic_curator._is_validated
-        if self._schema.itype == "Composite":
-            super().validate()
+        self._atomic_curator.validate()
+        self._is_validated = self._atomic_curator._is_validated
+        super().validate()
 
     @doc_args(SAVE_ARTIFACT_DOCSTRING)
     def save_artifact(
@@ -770,19 +775,13 @@ class DataFrameCurator(SlotsCurator):
         """{}."""
         if not self._is_validated:
             self.validate()
-
-        if self._slots:
-            self._slots["columns"] = self._atomic_curator
-            try:
-                return super().save_artifact(
-                    key=key, description=description, revises=revises, run=run
-                )
-            finally:
-                del self._slots["columns"]
-        else:
-            return self._atomic_curator.save_artifact(
+        self._slots["columns"] = self._atomic_curator
+        try:
+            return super().save_artifact(
                 key=key, description=description, revises=revises, run=run
             )
+        finally:
+            del self._slots["columns"]
 
 
 class ExperimentalDictCurator(DataFrameCurator):
@@ -1429,6 +1428,15 @@ class CatVector:
         # if a value is a list, we need to flatten it
         str_values = _flatten_unique(values)
 
+        # if values are SQLRecord, we don't need to validate them
+        if all(isinstance(v, SQLRecord) for v in str_values):
+            assert all(v._state.adding is False for v in str_values), (
+                "All records must be saved."
+            )
+            self.records = str_values  # type: ignore
+            validated_labels = str_values  # type: ignore
+            return validated_labels, []
+
         # inspect the default instance and save validated records from public
         if self._type_record is not None:
             related_name = registry._meta.get_field("type").remote_field.related_name
@@ -1972,7 +1980,7 @@ def annotate_artifact(
                 )
                 itype = (
                     Feature.name
-                    if artifact.schema.itype == "Composite"
+                    if artifact.schema.itype == "Composite"  # backward compat
                     else parse_cat_dtype(artifact.schema.itype, is_itype=True)["field"]
                 )
                 feature_set = Schema(itype=itype, n=len(features))
@@ -2011,7 +2019,8 @@ def annotate_artifact(
                 )
                 itype = (
                     Feature.name
-                    if artifact.schema.slots[slot].itype == "Composite"
+                    if artifact.schema.slots[slot].itype
+                    == "Composite"  # backward compat
                     else parse_cat_dtype(
                         artifact.schema.slots[slot].itype, is_itype=True
                     )["field"]
