@@ -1,8 +1,6 @@
 from __future__ import annotations
 
 import importlib
-import sys
-from pathlib import Path
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -17,14 +15,10 @@ def anndata_ensembl_gene_ids_and_valid_features_in_obs() -> Schema:
     """
     from ... import Schema
 
-    docs_path = Path(__file__).parent.parent.parent.parent / "docs" / "scripts"
-    if str(docs_path) not in sys.path:
-        sys.path.append(str(docs_path))
-
     try:
         return Schema.get(name="anndata_ensembl_gene_ids_and_valid_features_in_obs")
     except Schema.DoesNotExist:
-        import define_schema_anndata_ensembl_gene_ids_and_valid_features_in_obs  # noqa
+        from . import define_schema_anndata_ensembl_gene_ids_and_valid_features_in_obs  # noqa
 
         try:
             return Schema.get(name="anndata_ensembl_gene_ids_and_valid_features_in_obs")
