@@ -1,8 +1,6 @@
 from __future__ import annotations
 
 import importlib
-import sys
-from pathlib import Path
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -17,15 +15,11 @@ def valid_features() -> Schema:
     """
     from ... import Schema
 
-    docs_path = Path(__file__).parent.parent.parent.parent / "docs" / "scripts"
-    if str(docs_path) not in sys.path:
-        sys.path.append(str(docs_path))
-
     try:
         return Schema.get(name="valid_features")
     except Schema.DoesNotExist:
         try:
-            import define_valid_features  # noqa
+            from . import define_valid_features  # noqa
 
             return Schema.get(name="valid_features")
         except Schema.DoesNotExist:
