@@ -23,10 +23,9 @@ from ._describe import (
     NAME_WIDTH,
     TYPE_WIDTH,
     VALUES_WIDTH,
-    describe_header,
     format_rich_tree,
 )
-from ._django import get_artifact_with_related, get_related_model
+from ._django import get_artifact_or_run_with_related, get_related_model
 from ._relations import dict_related_model_to_related_name
 
 if TYPE_CHECKING:
@@ -65,29 +64,27 @@ def _get_labels_postgres(
     This is a postgres-specific approach that uses django Subquery.
     """
     if m2m_data is None:
-        artifact_meta = get_artifact_with_related(self, include_m2m=True)
+        artifact_meta = get_artifact_or_run_with_related(self, include_m2m=True)
         m2m_data = artifact_meta.get("related_data", {}).get("m2m", {})
     return m2m_data
 
 
 def describe_labels(
     self: Artifact | Collection,
-    labels_data: dict | None = None,
-    tree: Tree | None = None,
-    as_subtree: bool = False,
-):
-    """Describe labels associated with an artifact or collection."""
-    if not self._state.adding and connections[self._state.db].vendor == "postgresql":
-        labels_data = _get_labels_postgres(self, labels_data)
+    related_data: dict | None = None,
+) -> Tree | None:
+    """Describe labels."""
+    labels_data = related_data.get("m2m") if related_data is not None else None
+    if labels_data is None:
+        if (
+            not self._state.adding
+            and connections[self._state.db].vendor == "postgresql"
+        ):
+            labels_data = _get_labels_postgres(self, labels_data)
+        if not labels_data:
+            labels_data = _get_labels(self, instance=self._state.db)
     if not labels_data:
-        labels_data = _get_labels(self, instance=self._state.db)
-
-    # initialize tree
-    if tree is None:
-        tree = describe_header(self)
-    if not labels_data:
-        return tree
-
+        return None
     labels_table = Table(
         Column("", style="", no_wrap=True, width=NAME_WIDTH),
         Column("", style="dim", no_wrap=True, width=TYPE_WIDTH),
@@ -99,8 +96,14 @@ def describe_labels(
     for related_name, labels in labels_data.items():
         if not labels or related_name == "feature_sets":
             continue
-        if isinstance(labels, dict):  # postgres, labels are a dict[id, name]
-            print_values = _format_values(labels.values(), n=10, quotes=False)
+        if isinstance(labels, dict):
+            displays = [
+                d[key]
+                for d in labels.values()
+                for key in d.keys()
+                if key.endswith("_display")
+            ]
+            print_values = _format_values(displays, n=10, quotes=False)
         else:  # labels are a QuerySet
             field = get_name_field(labels)
             print_values = _format_values(
@@ -112,18 +115,11 @@ def describe_labels(
             labels_table.add_row(
                 f".{related_name}", Text(type_str, style="dim"), print_values
             )
-
-    labels_header = Text("Labels", style="bold green_yellow")
-    if as_subtree:
-        if labels_table.rows:
-            labels_tree = Tree(labels_header, guide_style="dim")
-            labels_tree.add(labels_table)
-            return labels_tree
-    else:
-        if labels_table.rows:
-            labels_tree = tree.add(labels_header)
-            labels_tree.add(labels_table)
-        return tree
+    tree = None
+    if labels_table.rows:  # we might not have rows even if labels_data was non-empty
+        tree = Tree(Text("Labels", style="bold green_yellow"), guide_style="dim")
+        tree.add(labels_table)
+    return tree
 
 
 def _save_validated_records(
@@ -228,13 +224,15 @@ class LabelManager:
         """Add labels from an artifact or collection to another artifact or collection.
 
         Examples:
-            >>> artifact1 = ln.Artifact(pd.DataFrame(index=[0, 1])).save()
-            >>> artifact2 = ln.Artifact(pd.DataFrame(index=[2, 3])).save()
-            >>> ulabels = ln.ULabel.from_values(["Label1", "Label2"], field="name")
-            >>> ln.save(ulabels)
-            >>> labels = ln.ULabel.filter(name__icontains = "label")
-            >>> artifact1.ulabels.set(labels)
-            >>> artifact2.labels.add_from(artifact1)
+
+            ::
+
+                artifact1 = ln.Artifact(pd.DataFrame(index=[0, 1])).save()
+                artifact2 = ln.Artifact(pd.DataFrame(index=[2, 3])).save()
+                records = ln.Record.from_values(["Label1", "Label2"], field="name").save()
+                labels = ln.Record.filter(name__icontains = "label")
+                artifact1.records.set(labels)
+                artifact2.labels.add_from(artifact1)
         """
         if transfer_logs is None:
             transfer_logs = {"mapped": [], "transferred": [], "run": None}
