@@ -25,7 +25,6 @@ from ..models._is_versioned import process_revises
 from ._is_versioned import IsVersioned
 from .artifact import (
     Artifact,
-    describe_artifact_collection,
     get_run,
     populate_subsequent_run,
     save_schema_links,
@@ -56,46 +55,10 @@ if TYPE_CHECKING:
     from .ulabel import ULabel
 
 
-# below is a draft for the future, see also the tests in test_collection.py
-#
-# class CollectionFeatureManager:
-#     """Query features of artifact in collection."""
-
-#     def __init__(self, collection: Collection):
-#         self._collection = collection
-
-#     def _get_staged_feature_sets_union(self) -> dict[str, Schema]:
-#         links_schema_artifact = Artifact.feature_sets.through.objects.filter(
-#             artifact_id__in=self._collection.artifacts.values_list("id", flat=True)
-#         )
-#         feature_sets_by_slots = defaultdict(list)
-#         for link in links_schema_artifact:
-#             feature_sets_by_slots[link.slot].append(link.schema_id)
-#         feature_sets_union = {}
-#         for slot, schema_ids_slot in feature_sets_by_slots.items():
-#             schema_1 = Schema.get(id=schema_ids_slot[0])
-#             related_name = schema_1._get_related_name()
-#             features_registry = getattr(Schema, related_name).field.model
-#             # this way of writing the __in statement turned out to be the fastest
-#             # evaluated on a link table with 16M entries connecting 500 feature sets with
-#             # 60k genes
-#             feature_ids = (
-#                 features_registry.schemas.through.objects.filter(
-#                     schema_id__in=schema_ids_slot
-#                 )
-#                 .values(f"{features_registry.__name__.lower()}_id")
-#                 .distinct()
-#             )
-#             features = features_registry.filter(id__in=feature_ids)
-#             feature_sets_union[slot] = Schema(features, dtype=schema_1.dtype)
-#         return feature_sets_union
-
-
 def _load_concat_artifacts(
     artifacts: list[Artifact], join: Literal["inner", "outer"] = "outer", **kwargs
 ) -> pd.DataFrame | ad.AnnData:
     suffixes = {artifact.suffix for artifact in artifacts}
-    # Why is that? - Sergei
     if len(suffixes) != 1:
         raise ValueError(
             "Can only load collections where all artifacts have the same suffix"
@@ -142,13 +105,13 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
 
     Examples:
 
-        Create a collection from a list of :class:`~lamindb.Artifact` objects:
+        Create a collection from a list of :class:`~lamindb.Artifact` objects::
 
-        >>> collection = ln.Collection([artifact1, artifact2], key="my_project/my_collection")
+            collection = ln.Collection([artifact1, artifact2], key="my_project/my_collection")
 
-        Create a collection that groups a data & a metadata artifact (e.g., here :doc:`docs:rxrx`):
+        Create a collection that groups a data & a metadata artifact (e.g., here :doc:`docs:rxrx`)::
 
-        >>> collection = ln.Collection(data_artifact, key="my_project/my_collection", meta=metadata_artifact)
+            collection = ln.Collection(data_artifact, key="my_project/my_collection", meta=metadata_artifact)
 
     """
 
@@ -327,10 +290,15 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
                     )
         # we ignore collections in trash containing the same hash
         if hash is not None and not skip_hash_lookup:
+            # this purposefully leaves out the key that we have
+            # in the hard database unique constraint
+            # so that the user is able to find collections with the same hash across
+            # keys
+            # if this is not desired, set skip_hash_lookup=True
             existing_collection = Collection.objects.filter(
                 ~Q(branch_id=-1),
                 hash=hash,
-            ).one_or_none()
+            ).first()
         else:
             existing_collection = None
         if existing_collection is not None:
@@ -380,7 +348,7 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
             expressions: Fields and values passed as Django query expressions.
 
         Raises:
-            :exc:`docs:lamindb.errors.DoesNotExist`: In case no matching record is found.
+            :exc:`lamindb.errors.DoesNotExist`: In case no matching record is found.
 
         See Also:
             - Method in `SQLRecord` base class: :meth:`~lamindb.models.SQLRecord.get`
@@ -668,13 +636,15 @@ class Collection(SQLRecord, IsVersioned, TracksRun, TracksUpdates):
         """
         return self.artifacts.first()
 
-    def describe(self) -> None:
-        """Describe relations of record.
+    def describe(self, return_str: bool = False) -> None | str:
+        """Describe record including relations.
 
-        Examples:
-            >>> artifact.describe()
+        Args:
+            return_str: Return a string instead of printing.
         """
-        return describe_artifact_collection(self)
+        from ._describe import describe_postgres_sqlite
+
+        return describe_postgres_sqlite(self, return_str=return_str)
 
 
 # internal function, not exposed to user
@@ -689,9 +659,8 @@ def from_artifacts(artifacts: Iterable[Artifact]) -> tuple[str, dict[str, str]]:
     if len(hashes) != len(hashes_set):
         seen = set()
         non_unique = [x for x in hashes if x in seen or seen.add(x)]  # type: ignore
-        raise ValueError(
-            "Please pass artifacts with distinct hashes: these ones are non-unique"
-            f" {non_unique}"
+        logger.warning(
+            f"your collection contains artifacts with non-unique hashes:  {non_unique}"
         )
     hash = hash_set(hashes_set)
     return hash
