@@ -7,7 +7,7 @@ from typing import TYPE_CHECKING, Literal
 import lamindb_setup as ln_setup
 from lamin_utils import logger
 
-from .query_set import QuerySet, SQLRecordList
+from .query_set import SQLRecordList, get_default_branch_ids
 from .run import Run
 from .sqlrecord import format_field_value, get_name_field
 
@@ -18,46 +18,53 @@ if TYPE_CHECKING:
 
     from .artifact import Artifact
     from .collection import Collection
+    from .query_set import BasicQuerySet, QuerySet
     from .sqlrecord import SQLRecord
-    from .transform import Transform
 
 LAMIN_GREEN_LIGHTER = "#10b981"
 LAMIN_GREEN_DARKER = "#065f46"
+TRANSFORM_VIOLET = "#eff2ff"
 GREEN_FILL = "honeydew"
-TRANSFORM_EMOJIS = {
-    "notebook": "📔",
-    "upload": "🖥️",
-    "pipeline": "🧩",
-    "script": "📝",
-    "function": "🔧",
-    "linker": "🧲",
-}
 is_run_from_ipython = getattr(builtins, "__IPYTHON__", False)
 
 
 # this is optimized to have fewer recursive calls
 # also len of QuerySet can be costly at times
 def _query_relatives(
-    records: QuerySet | list[SQLRecord],
-    attr: str,
-    cls: type[HasParents],
+    records: BasicQuerySet | list[HasParents],
+    attr: Literal["children", "parents", "records"],
 ) -> QuerySet:
-    from .query_set import get_default_branch_ids
-
     branch_ids = get_default_branch_ids()
 
-    def query_relatives_on_branches(records, attr, cls) -> QuerySet:
-        relatives = cls.objects.none()  # type: ignore
-        if len(records) == 0:
-            return relatives
-        for record in records:
-            relatives = relatives.union(
-                getattr(record, attr).filter(branch_id__in=branch_ids)
-            )
-        relatives = relatives.union(query_relatives_on_branches(relatives, attr, cls))
-        return relatives
+    if hasattr(records, "values_list"):
+        model = records.model  # type: ignore
+        frontier_ids = set(records.values_list("id", flat=True))
+    else:
+        model = records[0].__class__
+        frontier_ids = {r.id for r in records}  # type: ignore
 
-    return query_relatives_on_branches(records, attr, cls)
+    if attr == "records":
+        attr_filter = "type__id__in"
+    elif attr == "children":
+        attr_filter = "parents__id__in"
+    else:
+        attr_filter = "children__id__in"
+
+    seen_ids = set(frontier_ids)  # copies
+    results = set()
+
+    while frontier_ids:
+        relatives_qs = model.filter(
+            branch_id__in=branch_ids, **{attr_filter: frontier_ids}
+        )
+        next_ids = set(relatives_qs.values_list("id", flat=True)) - seen_ids
+        if not next_ids:
+            break
+        results.update(next_ids)
+        seen_ids.update(next_ids)
+        frontier_ids = next_ids
+
+    return model.filter(id__in=results)
 
 
 def _query_ancestors_of_fk(record: SQLRecord, attr: str) -> SQLRecordList:
@@ -147,18 +154,11 @@ class HasParents:
 
     def query_parents(self) -> QuerySet:
         """Query parents in an ontology."""
-        return _query_relatives([self], "parents", self.__class__)  # type: ignore
+        return _query_relatives([self], "parents")  # type: ignore
 
     def query_children(self) -> QuerySet:
         """Query children in an ontology."""
-        return _query_relatives([self], "children", self.__class__)  # type: ignore
-
-
-def _transform_emoji(transform: Transform):
-    if transform is not None:
-        return TRANSFORM_EMOJIS.get(transform.type, "💫")
-    else:
-        return TRANSFORM_EMOJIS["pipeline"]
+        return _query_relatives([self], "children")  # type: ignore
 
 
 def view_digraph(u: Digraph):
@@ -205,8 +205,6 @@ def view_lineage(
         df_values += _get_all_child_runs(data)
     df_edges = _df_edges_from_runs(df_values)
 
-    data_label = _record_label(data)
-
     def add_node(
         record: Run | Artifact | Collection,
         node_id: str,
@@ -214,9 +212,9 @@ def view_lineage(
         u: graphviz.Digraph,
     ):
         if isinstance(record, Run):
-            fillcolor = "gainsboro"
+            fillcolor = TRANSFORM_VIOLET
         else:
-            fillcolor = GREEN_FILL
+            fillcolor = "white"
         u.node(
             node_id,
             label=node_label,
@@ -228,8 +226,8 @@ def view_lineage(
     u = graphviz.Digraph(
         f"{data._meta.model_name}_{data.uid}",
         node_attr={
-            "fillcolor": GREEN_FILL,
-            "color": LAMIN_GREEN_DARKER,
+            "fillcolor": "white",
+            "color": "darkgrey",
             "fontname": "Helvetica",
             "fontsize": "10",
         },
@@ -242,12 +240,12 @@ def view_lineage(
             add_node(row["target_record"], row["target"], row["target_label"], u)
 
         u.edge(row["source"], row["target"], color="dimgrey")
-    # label the searched file
+
     u.node(
         f"{data._meta.model_name}_{data.uid}",
-        label=data_label,
+        label=get_record_label(data),
         style="rounded,filled",
-        fillcolor=LAMIN_GREEN_LIGHTER,
+        fillcolor="white",
         shape="box",
     )
 
@@ -315,8 +313,6 @@ def view_parents(
     else:
         return None
 
-    record_label = _record_label(record, field)
-
     u = graphviz.Digraph(
         record.uid,
         node_attr={
@@ -331,11 +327,7 @@ def view_parents(
     )
     u.node(
         record.uid,
-        label=(
-            _record_label(record)
-            if record.__class__.__name__ == "Transform"
-            else _add_emoji(record, record_label)
-        ),
+        label=(get_record_label(record)),
         fillcolor=LAMIN_GREEN_LIGHTER,
     )
     if df_edges is not None:
@@ -420,81 +412,50 @@ def _df_edges_from_parents(
     df_edges["source_record"] = df_edges["source"].apply(lambda x: all.get(id=x))
     df_edges["target_record"] = df_edges["target"].apply(lambda x: all.get(id=x))
     if record.__class__.__name__ == "Transform":
-        df_edges["source_label"] = df_edges["source_record"].apply(_record_label)
-        df_edges["target_label"] = df_edges["target_record"].apply(_record_label)
+        df_edges["source_label"] = df_edges["source_record"].apply(get_record_label)
+        df_edges["target_label"] = df_edges["target_record"].apply(get_record_label)
     else:
         df_edges["source_label"] = df_edges["source_record"].apply(
-            lambda x: _record_label(x, field)
+            lambda x: get_record_label(x, field)
         )
         df_edges["target_label"] = df_edges["target_record"].apply(
-            lambda x: _record_label(x, field)
+            lambda x: get_record_label(x, field)
         )
     df_edges["source"] = df_edges["source_record"].apply(lambda x: x.uid)
     df_edges["target"] = df_edges["target_record"].apply(lambda x: x.uid)
     return df_edges
 
 
-def _record_label(record: SQLRecord, field: str | None = None):
+def get_record_label(record: SQLRecord, field: str | None = None):
     from .artifact import Artifact
     from .collection import Collection
     from .transform import Transform
 
     if isinstance(record, Artifact):
-        if record.description is None:
-            name = record.key
-        else:
-            name = record.description.replace("&", "&amp;")
-
-        return (
-            rf'<📄 {name}<BR/><FONT COLOR="GREY" POINT-SIZE="10"'
-            rf' FACE="Monospace">uid={record.uid}<BR/>suffix={record.suffix}</FONT>>'
+        title = (
+            record.key.replace("&", "&amp;") if record.key is not None else record.uid
         )
+        return rf"<{title}>"
     elif isinstance(record, Collection):
-        name = record.name.replace("&", "&amp;")
+        title = record.key.replace("&", "&amp;")
         return (
-            rf'<🍱 {name}<BR/><FONT COLOR="GREY" POINT-SIZE="10"'
-            rf' FACE="Monospace">uid={record.uid}<BR/>version={record.version}</FONT>>'
+            rf'<{title}<BR/><FONT COLOR="GREY" POINT-SIZE="10"'
+            rf' FACE="Monospace">version={record.version}</FONT>>'
         )
     elif isinstance(record, Run):
-        if record.transform.description:
-            name = f"{record.transform.description.replace('&', '&amp;')}"
-        elif record.transform.key:
-            name = f"{record.transform.key.replace('&', '&amp;')}"
-        else:
-            name = f"{record.transform.uid}"
-        user_display = (
-            record.created_by.handle
-            if record.created_by.name is None
-            else record.created_by.name
-        )
+        title = record.transform.key.replace("&", "&amp;")
         return (
-            rf'<{TRANSFORM_EMOJIS.get(str(record.transform.type), "💫")} {name}<BR/><FONT COLOR="GREY" POINT-SIZE="10"'
-            rf' FACE="Monospace">uid={record.transform.uid}<BR/>type={record.transform.type},'
-            rf" user={user_display}<BR/>run={format_field_value(record.started_at)}</FONT>>"
+            rf'<{title}<BR/><FONT COLOR="GREY" POINT-SIZE="10">'
+            rf"run at {format_field_value(record.started_at)}</FONT>>"
         )
     elif isinstance(record, Transform):
-        name = f"{record.name.replace('&', '&amp;')}"
-        return (
-            rf'<{TRANSFORM_EMOJIS.get(str(record.type), "💫")} {name}<BR/><FONT COLOR="GREY" POINT-SIZE="10"'
-            rf' FACE="Monospace">uid={record.uid}<BR/>type={record.type},'
-            rf" user={record.created_by.name}<BR/>updated_at={format_field_value(record.updated_at)}</FONT>>"
-        )
+        title = record.key.replace("&", "&amp;")
+        return rf"<{title}>"
     else:
-        name = record.__getattribute__(field)
-        return (
-            rf'<{name}<BR/><FONT COLOR="GREY" POINT-SIZE="10"'
-            rf' FACE="Monospace">uid={record.uid}</FONT>>'
-        )
-
-
-def _add_emoji(record: SQLRecord, label: str):
-    if record.__class__.__name__ == "Transform":
-        emoji = TRANSFORM_EMOJIS.get(record.type, "💫")
-    elif record.__class__.__name__ == "Run":
-        emoji = TRANSFORM_EMOJIS.get(record.transform.type, "💫")
-    else:
-        emoji = ""
-    return f"{emoji} {label}"
+        if field is None:
+            field = get_name_field(record)
+        title = record.__getattribute__(field)
+        return rf"<{title}>"
 
 
 def _get_all_parent_runs(data: Artifact | Collection) -> list:
@@ -613,6 +574,6 @@ def _df_edges_from_runs(df_values: list):
     df = df.drop_duplicates().dropna()
     df["source"] = [f"{i._meta.model_name}_{i.uid}" for i in df["source_record"]]
     df["target"] = [f"{i._meta.model_name}_{i.uid}" for i in df["target_record"]]
-    df["source_label"] = df["source_record"].apply(_record_label)
-    df["target_label"] = df["target_record"].apply(_record_label)
+    df["source_label"] = df["source_record"].apply(get_record_label)
+    df["target_label"] = df["target_record"].apply(get_record_label)
     return df
