@@ -41,21 +41,31 @@ class Reference(SQLRecord, CanCurate, TracksRun, TracksUpdates, ValidateFields):
     """References such as internal studies, papers, documents, or URLs.
 
     Example:
-        >>> reference = Reference(
-        ...     name="A Paper Title",
-        ...     abbr="APT",
-        ...     url="https://doi.org/10.1000/xyz123",
-        ...     pubmed_id=12345678,
-        ...     doi="10.1000/xyz123",
-        ...     description="Good paper.",
-        ...     text="Some text I want to be searchable.",
-        ...     date=date(2023, 11, 21),
-        ... ).save()
+
+        ::
+            reference = Reference(
+                name="A Paper Title",
+                abbr="APT",
+                url="https://doi.org/10.1000/xyz123",
+                pubmed_id=12345678,
+                doi="10.1000/xyz123",
+                description="Good paper.",
+                text="Some text I want to be searchable.",
+                date=date(2023, 11, 21),
+            ).save()
     """
 
     class Meta(SQLRecord.Meta, TracksRun.Meta, TracksUpdates.Meta):
         abstract = False
         app_label = "lamindb"
+        constraints = [
+            models.UniqueConstraint(
+                fields=["name", "type", "space"],
+                name="unique_reference_name_type_space",
+                condition=~models.Q(branch_id=-1),
+            )
+            # also see raw SQL constraints for `is_type` and `type` FK validity in migrations
+        ]
 
     id: int = models.AutoField(primary_key=True)
     """Internal id, valid only in one DB instance."""
@@ -99,7 +109,7 @@ class Reference(SQLRecord, CanCurate, TracksRun, TracksUpdates, ValidateFields):
         ],
     )
     """Digital Object Identifier (DOI) for the reference."""
-    text: str | None = TextField(null=True, db_index=True)
+    text: str | None = TextField(null=True)
     """Abstract or full text of the reference to make it searchable."""
     date: DateType | None = DateField(null=True, default=None)
     """Date of creation or publication of the reference."""
@@ -107,11 +117,11 @@ class Reference(SQLRecord, CanCurate, TracksRun, TracksUpdates, ValidateFields):
         Artifact, through="ArtifactReference", related_name="references"
     )
     """Annotated artifacts."""
-    transforms: Artifact = models.ManyToManyField(
+    transforms: Transform = models.ManyToManyField(
         Transform, through="TransformReference", related_name="references"
     )
     """Annotated transforms."""
-    collections: Artifact = models.ManyToManyField(
+    collections: Collection = models.ManyToManyField(
         Collection, through="CollectionReference", related_name="references"
     )
     """Annotated collections."""
@@ -152,23 +162,33 @@ class Reference(SQLRecord, CanCurate, TracksRun, TracksUpdates, ValidateFields):
 
 
 class Project(SQLRecord, CanCurate, TracksRun, TracksUpdates, ValidateFields):
-    """Projects to label artifacts, transforms, and runs.
+    """Projects to label artifacts, transforms, records, and runs.
 
-    Example::
+    Example:
 
-        project = Project(
-           name="My Project Name",
-           abbr="MPN",
-           url="https://example.com/my_project",
-        ).save()
-        artifact.projects.add(project)  # <-- labels the artifact with the project
-        ln.track(project=project)       # <-- automtically labels entities during the run
+        ::
+
+            project = Project(
+            name="My Project Name",
+            abbr="MPN",
+            url="https://example.com/my_project",
+            ).save()
+            artifact.projects.add(project)  # <-- labels the artifact with the project
+            ln.track(project=project)       # <-- automtically labels entities during the run
 
     """
 
     class Meta(SQLRecord.Meta, TracksRun.Meta, TracksUpdates.Meta):
         abstract = False
         app_label = "lamindb"
+        constraints = [
+            models.UniqueConstraint(
+                fields=["name", "type", "space"],
+                name="unique_project_name_type_space",
+                condition=~models.Q(branch_id=-1),
+            )
+            # also see raw SQL constraints for `is_type` and `type` FK validity in migrations
+        ]
 
     id: int = models.AutoField(primary_key=True)
     """Internal id, valid only in one DB instance."""
@@ -230,11 +250,11 @@ class Project(SQLRecord, CanCurate, TracksRun, TracksUpdates, ValidateFields):
         ULabel, through="ULabelProject", related_name="projects"
     )
     """Annotated ulabels."""
-    features: ULabel = models.ManyToManyField(
+    features: Feature = models.ManyToManyField(
         Feature, through="FeatureProject", related_name="projects"
     )
     """Annotated features."""
-    schemas: ULabel = models.ManyToManyField(
+    schemas: Schema = models.ManyToManyField(
         Schema, through="SchemaProject", related_name="projects"
     )
     """Annotated schemas."""
