@@ -4,7 +4,6 @@ import re
 from collections import UserList
 from collections.abc import Iterable
 from collections.abc import Iterable as IterableType
-from datetime import datetime, timezone
 from typing import TYPE_CHECKING, Any, Generic, NamedTuple, TypeVar
 
 import pandas as pd
@@ -34,12 +33,6 @@ T = TypeVar("T")
 pd.set_option("display.max_columns", 200)
 
 
-# def format_and_convert_to_local_time(series: pd.Series):
-#     tzinfo = datetime.now().astimezone().tzinfo
-#     timedelta = tzinfo.utcoffset(datetime.now())  # type: ignore
-#     return (series + timedelta).dt.strftime("%Y-%m-%d %H:%M:%S %Z")
-
-
 def get_keys_from_df(data: list, registry: SQLRecord) -> list[str]:
     if len(data) > 0:
         if isinstance(data[0], dict):
@@ -342,34 +335,34 @@ def get_basic_field_names(
     features_input: bool | list[str] | str,
 ) -> list[str]:
     exclude_field_names = ["updated_at"]
+    include_private_fields = False
+    if "privates" in include:
+        include_private_fields = True
+        include.remove("privates")
     field_names = [
         field.name
         for field in qs.model._meta.fields
         if (
             not isinstance(field, models.ForeignKey)
             and field.name not in exclude_field_names
+            and (not field.name.startswith("_") or include_private_fields)
         )
     ]
-    field_names += [
-        f"{field.name}_id"
-        for field in qs.model._meta.fields
-        if isinstance(field, models.ForeignKey)
-    ]
     for field_name in [
         "version",
         "is_latest",
         "is_locked",
-        "run_id",
         "created_at",
-        "created_by_id",
         "updated_at",
-        "_aux",
-        "_real_key",
-        "branch_id",
     ]:
         if field_name in field_names:
             field_names.remove(field_name)
             field_names.append(field_name)
+    field_names += [
+        f"{field.name}_id"
+        for field in qs.model._meta.fields
+        if isinstance(field, models.ForeignKey)
+    ]
     if field_names[0] != "uid" and "uid" in field_names:
         field_names.remove("uid")
         field_names.insert(0, "uid")
@@ -393,6 +386,7 @@ def get_feature_annotate_kwargs(
         Feature,
         Record,
         RecordJson,
+        ULabel,
     )
 
     if registry not in {Artifact, Record}:
@@ -404,13 +398,27 @@ def get_feature_annotate_kwargs(
         ids_list = qs.values_list("id", flat=True)
         feature_names = []
         for obj in registry._meta.related_objects:
-            if not hasattr(getattr(registry, obj.related_name), "through"):
+            related_name_attr = getattr(registry, obj.related_name, None)
+            if related_name_attr is None or not hasattr(related_name_attr, "through"):
                 continue
-            link_model = getattr(registry, obj.related_name).through
-            if link_model.__name__ == "Record_parents":
+            link_model = related_name_attr.through
+            if (
+                not hasattr(link_model, "feature")
+                or link_model.__name__ == "Record_parents"
+            ):
                 continue
-            links = link_model.filter(
-                **{registry.__name__.lower() + "_id__in": ids_list}
+            filter_field = registry.__name__.lower()
+            if not hasattr(link_model, filter_field):
+                potential_fields = []
+                for field in link_model._meta.get_fields():
+                    if field.is_relation and field.related_model is registry:
+                        potential_fields.append(field.name)
+                if len(potential_fields) == 1:
+                    filter_field = potential_fields[0]
+                else:
+                    continue
+            links = link_model.objects.using(qs.db).filter(
+                **{filter_field + "_id__in": ids_list}
             )
             feature_names_for_link_model = links.values_list("feature__name", flat=True)
             feature_names += feature_names_for_link_model
@@ -422,19 +430,21 @@ def get_feature_annotate_kwargs(
             )
         features = list(set(feature_names))  # remove duplicates
 
-    feature_qs = Feature.filter()
+    feature_qs = Feature.connect(None if qs is None else qs.db).filter(
+        dtype__isnull=False
+    )
     if isinstance(features, list):
         feature_qs = feature_qs.filter(name__in=features)
         feature_names = features
     else:  # features is True -- only consider categorical features from ULabel and non-categorical features
         feature_qs = feature_qs.filter(
-            Q(~Q(dtype__startswith="cat["))
+            ~Q(dtype__startswith="cat[")
             | Q(dtype__startswith="cat[ULabel")
             | Q(dtype__startswith="cat[Record")
         )
         feature_names = feature_qs.to_list("name")
         logger.important(
-            f"queried for all categorical features with dtype ULabel or Record and non-categorical features: ({len(feature_names)}) {feature_names}"
+            f"queried for all categorical features with dtype Record and non-categorical features: ({len(feature_names)}) {feature_names}"
         )
     # Get the categorical features
     cat_feature_types = {
@@ -446,14 +456,14 @@ def get_feature_annotate_kwargs(
     link_models_on_models = {
         getattr(
             registry, obj.related_name
-        ).through.__get_name_with_module__(): obj.related_model.__get_name_with_module__()
+        ).through.__get_name_with_module__(): obj.related_model
         for obj in registry._meta.related_objects
         if obj.related_model.__get_name_with_module__() in cat_feature_types
     }
     if registry is Artifact:
-        link_models_on_models["ArtifactULabel"] = "ULabel"
+        link_models_on_models["ArtifactULabel"] = ULabel
     else:
-        link_models_on_models["RecordRecord"] = "Record"
+        link_models_on_models["RecordRecord"] = Record
     link_attributes_on_models = {
         obj.related_name: link_models_on_models[
             obj.related_model.__get_name_with_module__()
@@ -470,7 +480,8 @@ def get_feature_annotate_kwargs(
     }
     # Prepare Django's annotate for features
     annotate_kwargs = {}
-    for link_attr, feature_type in link_attributes_on_models.items():
+    for link_attr, feature_type_model in link_attributes_on_models.items():
+        feature_type = feature_type_model.__get_name_with_module__()
         if link_attr == "links_project" and registry is Record:
             # we're only interested in _values_project when "annotating" records
             continue
@@ -484,7 +495,7 @@ def get_feature_annotate_kwargs(
         else:
             field_name = "value"
         annotate_kwargs[f"{link_attr}__{field_name}__name"] = F(
-            f"{link_attr}__{field_name}__name"
+            f"{link_attr}__{field_name}__{getattr(feature_type_model, '_name_field', 'name')}"
         )
     json_values_attribute = "_feature_values" if registry is Artifact else "values_json"
     annotate_kwargs[f"{json_values_attribute}__feature__name"] = F(
@@ -550,6 +561,7 @@ def analyze_lookup_cardinality(
 def reorder_subset_columns_in_df(
     df: pd.DataFrame, column_order: list[str], position=3
 ) -> pd.DataFrame:
+    """Reorder subset of columns in dataframe to specified position."""
     valid_columns = [col for col in column_order if col in df.columns]
     all_cols = df.columns.tolist()
     remaining_cols = [col for col in all_cols if col not in valid_columns]
@@ -557,6 +569,26 @@ def reorder_subset_columns_in_df(
     return df[new_order]
 
 
+def encode_lamindb_fields_as_columns(
+    registry: Registry, fields: str | list[str]
+) -> str | dict[str, str]:
+    """Encode laminDB specific fields in dataframe with __lamindb_{model_name}_{field_name}__.
+
+    This is needed when reshaping dataframes with features to avoid conflicts between
+    laminDB fields and feature names.
+    """
+
+    def encode(field: str) -> str:
+        return f"__lamindb_{registry._meta.model_name}_{field}__"
+
+    registry_field_names = {field.name for field in registry._meta.concrete_fields}
+
+    if isinstance(fields, str):
+        return encode(fields) if fields in registry_field_names else fields
+
+    return {field: encode(field) for field in fields if field in registry_field_names}
+
+
 # https://lamin.ai/laminlabs/lamindata/transform/BblTiuKxsb2g0003
 # https://claude.ai/chat/6ea2498c-944d-4e7a-af08-29e5ddf637d2
 def reshape_annotate_result(
@@ -570,107 +602,141 @@ def reshape_annotate_result(
     """Reshapes tidy table to wide format.
 
     Args:
-        field_names: List of basic fields to include in result
+        registry: The registry model (e.g., Artifact)
         df: Input dataframe with experimental data
-        extra_columns: Dict specifying additional columns to process with types ('one' or 'many')
-            e.g., {'ulabels__name': 'many', 'created_by__name': 'one'}
-        feature_names: Feature names.
+        field_names: List of basic fields to include in result
+        cols_from_include: Dict specifying additional columns to process with types
+            ('one' or 'many'), e.g., {'ulabels__name': 'many', 'created_by__name': 'one'}
+        feature_names: Feature names to include
+        feature_qs: QuerySet of features for type conversion
     """
     from lamindb.models import Artifact
 
     cols_from_include = cols_from_include or {}
 
-    json_values_attribute = "_feature_values" if registry is Artifact else "values_json"
-
-    # initialize result with basic fields, need a copy as we're modifying it
-    # will give us warnings otherwise
+    # Initialize result with basic fields (need a copy since we're modifying it)
     result = df[field_names].copy()
-    # process features if requested
-    if feature_names:
-        # handle json values
-        feature_cols = [
-            f"{json_values_attribute}__feature__name",
-            f"{json_values_attribute}__value",
-        ]
-        if all(col in df.columns for col in feature_cols):
-            # Create two separate dataframes - one for dict values and one for non-dict values
-            is_dict = df[f"{json_values_attribute}__value"].apply(
-                lambda x: isinstance(x, dict)
+    pk_name = registry._meta.pk.name
+
+    # ========== no features requested ==========
+    if not feature_names:
+        if cols_from_include:
+            result = process_cols_from_include(df, result, cols_from_include, pk_name)
+        return result.drop_duplicates(subset=[pk_name])
+
+    # ========== process features ==========
+
+    # Encode Django field names to avoid conflicts with feature names
+    fields_map = encode_lamindb_fields_as_columns(registry, df.columns)
+    df_encoded = df.rename(columns=fields_map)
+    result_encoded = result.rename(columns=fields_map)
+    pk_name_encoded = fields_map.get(pk_name)  # type: ignore
+
+    # --- Process JSON-stored feature values ---
+    json_values_attribute = "_feature_values" if registry is Artifact else "values_json"
+    feature_name_col = f"{json_values_attribute}__feature__name"
+    feature_value_col = f"{json_values_attribute}__value"
+
+    if all(col in df_encoded.columns for col in [feature_name_col, feature_value_col]):
+        # Separate dict and non-dict values for different aggregation strategies
+        is_dict = df_encoded[feature_value_col].apply(lambda x: isinstance(x, dict))
+        dict_df = df_encoded[is_dict]
+        non_dict_df = df_encoded[~is_dict]
+
+        # Aggregate: sets for non-dict values, first for dict values
+        groupby_cols = [pk_name_encoded, feature_name_col]
+        non_dict_features = non_dict_df.groupby(groupby_cols)[feature_value_col].agg(
+            set
+        )
+        dict_features = dict_df.groupby(groupby_cols)[feature_value_col].agg("first")
+
+        # Combine and pivot to wide format
+        combined_features = pd.concat([non_dict_features, dict_features])
+        feature_values = combined_features.unstack().reset_index()
+
+        if not feature_values.empty:
+            result_encoded = result_encoded.join(
+                feature_values.set_index(pk_name_encoded),
+                on=pk_name_encoded,
             )
-            dict_df, non_dict_df = df[is_dict], df[~is_dict]
 
-            # Process non-dict values using set aggregation
-            non_dict_features = non_dict_df.groupby(
-                ["id", f"{json_values_attribute}__feature__name"]
-            )[f"{json_values_attribute}__value"].agg(set)
+    # --- Process categorical/linked features ---
+    links_prefix = "links_" if registry is Artifact else ("links_", "values_")
+    links_features = [
+        col
+        for col in df.columns
+        if "feature__name" in col and col.startswith(links_prefix)
+    ]
 
-            # Process dict values using first aggregation
-            dict_features = dict_df.groupby(
-                ["id", f"{json_values_attribute}__feature__name"]
-            )[f"{json_values_attribute}__value"].agg("first")
+    if links_features:
+        result_encoded = process_links_features(
+            df_encoded, result_encoded, links_features, feature_names, pk_name_encoded
+        )
 
-            # Combine the results
-            combined_features = pd.concat([non_dict_features, dict_features])
+    # --- Apply type conversions based on feature metadata ---
+    def extract_single_element(value):
+        """Extract single element from set/collection if it contains exactly one item."""
+        if not hasattr(value, "__len__"):  # NaN or other scalar
+            return value
+        if len(value) != 1:
+            # TODO: Make this dependent on feature._expect_many
+            return value
+        return next(iter(value))
 
-            # Unstack and reset index
-            feature_values = combined_features.unstack().reset_index()
-            if not feature_values.empty:
-                result = result.join(
-                    feature_values.set_index("id"),
-                    on="id",
+    for feature in feature_qs:
+        if feature.name not in result_encoded.columns:
+            continue
+
+        # Extract single values from sets where appropriate
+        # TODO: Make dependent on feature._expect_many
+        result_encoded[feature.name] = result_encoded[feature.name].apply(
+            extract_single_element
+        )
+
+        # Convert to categorical dtype if specified
+        if feature.dtype.startswith("cat"):
+            try:
+                result_encoded[feature.name] = result_encoded[feature.name].astype(
+                    "category"
                 )
+            except (TypeError, ValueError):
+                pass  # Keep original if conversion fails
 
-        # handle categorical features
-        links_prefix = "links_" if registry is Artifact else ("links_", "values_")
-        links_features = [
-            col
-            for col in df.columns
-            if "feature__name" in col and col.startswith(links_prefix)
-        ]
-
-        if links_features:
-            result = process_links_features(df, result, links_features, feature_names)
-
-        def extract_single_element(s):
-            if not hasattr(s, "__len__"):  # is NaN or other scalar
-                return s
-            if len(s) != 1:
-                # TODO: below should depend on feature._expect_many
-                # logger.warning(
-                #     f"expected single value because `feature._expect_many is False` but got set {len(s)} elements: {s}"
-                # )
-                return s
-            return next(iter(s))
-
-        for feature in feature_qs:
-            if feature.name in result.columns:
-                # TODO: make dependent on feature._expect_many through
-                # lambda x: extract_single_element(x, feature)
-                result[feature.name] = result[feature.name].apply(
-                    extract_single_element
+        # Convert to datetime if specified
+        if feature.dtype.startswith("datetime"):
+            try:
+                result_encoded[feature.name] = pd.to_datetime(
+                    result_encoded[feature.name]
                 )
-                if feature.dtype.startswith("cat"):
-                    try:
-                        # Try to convert to category - this will fail if complex objects remain
-                        result[feature.name] = result[feature.name].astype("category")
-                    except (TypeError, ValueError):
-                        # If conversion fails, the column still contains complex objects
-                        pass
-                if feature.dtype.startswith("datetime"):
-                    try:
-                        # Try to convert to category - this will fail if complex objects remain
-                        result[feature.name] = pd.to_datetime(result[feature.name])
-                    except (TypeError, ValueError):
-                        # If conversion fails, the column still contains complex objects
-                        pass
+            except (TypeError, ValueError):
+                pass  # Keep original if conversion fails
 
-        # sort columns
-        result = reorder_subset_columns_in_df(result, feature_names)
+    # --- Finalize result ---
 
+    # Reorder columns to prioritize features
+    result_encoded = reorder_subset_columns_in_df(result_encoded, feature_names)
+
+    # Process additional included columns
     if cols_from_include:
-        result = process_cols_from_include(df, result, cols_from_include)
+        cols_from_include_encoded = {
+            fields_map.get(k, k): v  # type: ignore
+            for k, v in cols_from_include.items()
+        }
+        result_encoded = process_cols_from_include(
+            df_encoded, result_encoded, cols_from_include_encoded, pk_name_encoded
+        )
 
-    return result.drop_duplicates(subset=["id"])
+    # Decode field names back to original, except where conflicts exist
+    # (e.g., if a feature is also named 'id', keep the encoded field name)
+    decode_map = {
+        encoded: original
+        for original, encoded in fields_map.items()  # type: ignore
+        if original not in result_encoded.columns
+    }
+
+    return result_encoded.drop_duplicates(subset=[pk_name_encoded]).rename(
+        columns=decode_map
+    )
 
 
 def process_links_features(
@@ -678,6 +744,7 @@ def process_links_features(
     result: pd.DataFrame,
     feature_cols: list[str],
     features: bool | list[str],
+    pk_name: str = "id",
 ) -> pd.DataFrame:
     """Process links_XXX feature columns."""
     # this loops over different entities that might be linked under a feature
@@ -704,17 +771,19 @@ def process_links_features(
         # Filter features if specific ones requested
         if isinstance(features, list):
             feature_names = [f for f in feature_names if f in features]
-
         for feature_name in feature_names:
             mask = df[feature_col] == feature_name
-            feature_values = df[mask].groupby("id")[value_col].agg(set)
-            result.insert(3, feature_name, result["id"].map(feature_values))
+            feature_values = df[mask].groupby(pk_name)[value_col].agg(set)
+            result.insert(3, feature_name, result[pk_name].map(feature_values))
 
     return result
 
 
 def process_cols_from_include(
-    df: pd.DataFrame, result: pd.DataFrame, extra_columns: dict[str, str]
+    df: pd.DataFrame,
+    result: pd.DataFrame,
+    extra_columns: dict[str, str],
+    pk_name: str = "id",
 ) -> pd.DataFrame:
     """Process additional columns based on their specified types."""
     for col, col_type in extra_columns.items():
@@ -723,8 +792,8 @@ def process_cols_from_include(
         if col in result.columns:
             continue
 
-        values = df.groupby("id")[col].agg(set if col_type == "many" else "first")
-        result.insert(3, col, result["id"].map(values))
+        values = df.groupby(pk_name)[col].agg(set if col_type == "many" else "first")
+        result.insert(3, col, result[pk_name].map(values))
 
     return result
 
@@ -746,7 +815,6 @@ def _queryset_class_factory(
         )
     else:
         new_cls = queryset_cls
-
     return new_cls
 
 
@@ -786,69 +854,101 @@ class BasicQuerySet(models.QuerySet):
     @doc_args(SQLRecord.to_dataframe.__doc__)
     def to_dataframe(
         self,
+        *,
         include: str | list[str] | None = None,
-        features: bool | list[str] | str | None = None,
+        features: str | list[str] | None = None,
+        limit: int | None = 100,
+        order_by: str | None = "-id",
     ) -> pd.DataFrame:
         """{}"""  # noqa: D415
-        time = datetime.now(timezone.utc)
+        # check if queryset is already ordered
+        is_ordered = bool(self.query.order_by)
+        # Only apply order_by if not already ordered and order_by is specified
+        if not is_ordered and order_by is not None:
+            subset = self.order_by(order_by)
+        else:
+            subset = self
+        if limit is not None:
+            subset = subset[:limit]
         if include is None:
             include_input = []
         elif isinstance(include, str):
             include_input = [include]
         else:
             include_input = include
+        if "features" in include_input:
+            include_input.remove("features")
+            if features is None:
+                # indicate the default features with True
+                # should refactor this in the future
+                features = True  # type: ignore
         features_input = [] if features is None else features
-        include = get_backward_compat_filter_kwargs(self, include_input)
-        field_names = get_basic_field_names(self, include_input, features_input)
+        include = get_backward_compat_filter_kwargs(subset, include_input)
+        field_names = get_basic_field_names(subset, include_input, features_input)
 
         annotate_kwargs = {}
         feature_names: list[str] = []
         feature_qs = None
         if features:
             feature_annotate_kwargs, feature_names, feature_qs = (
-                get_feature_annotate_kwargs(self.model, features, self)
+                get_feature_annotate_kwargs(subset.model, features, subset)
             )
-            time = logger.debug("finished feature_annotate_kwargs", time=time)
             annotate_kwargs.update(feature_annotate_kwargs)
         if include_input:
             include_input = include_input.copy()[::-1]  # type: ignore
             include_kwargs = {s: F(s) for s in include_input if s not in field_names}
             annotate_kwargs.update(include_kwargs)
         if annotate_kwargs:
-            id_subquery = self.values("id")
-            time = logger.debug("finished get id values", time=time)
+            id_subquery = subset.values("id")
             # for annotate, we want the queryset without filters so that joins don't affect the annotations
-            query_set_without_filters = self.model.objects.using(self._db).filter(
+            query_set_without_filters = subset.model.objects.using(subset.db).filter(
                 id__in=Subquery(id_subquery)
             )
-            time = logger.debug("finished get query_set_without_filters", time=time)
-            if self.query.order_by:
+            if subset.query.order_by:
                 # Apply the same ordering to the new queryset
                 query_set_without_filters = query_set_without_filters.order_by(
-                    *self.query.order_by
+                    *subset.query.order_by
                 )
-                time = logger.debug("finished order by", time=time)
             queryset = query_set_without_filters.annotate(**annotate_kwargs)
-            time = logger.debug("finished annotate", time=time)
         else:
-            queryset = self
+            queryset = subset
 
         df = pd.DataFrame(queryset.values(*field_names, *list(annotate_kwargs.keys())))
         if len(df) == 0:
             df = pd.DataFrame({}, columns=field_names)
             return df
-        time = logger.debug("finished creating first dataframe", time=time)
         cols_from_include = analyze_lookup_cardinality(self.model, include_input)  # type: ignore
-        time = logger.debug("finished analyze_lookup_cardinality", time=time)
         df_reshaped = reshape_annotate_result(
             self.model, df, field_names, cols_from_include, feature_names, feature_qs
         )
-        time = logger.debug("finished reshape_annotate_result", time=time)
         pk_name = self.model._meta.pk.name
-        pk_column_name = pk_name if pk_name in df.columns else f"{pk_name}_id"
-        if pk_column_name in df_reshaped.columns:
-            df_reshaped = df_reshaped.set_index(pk_column_name)
-        time = logger.debug("finished", time=time)
+        encoded_pk_name = encode_lamindb_fields_as_columns(self.model, pk_name)
+        if encoded_pk_name in df_reshaped.columns:
+            df_reshaped = df_reshaped.set_index(encoded_pk_name)
+        else:
+            pk_column_name = pk_name if pk_name in df.columns else f"{pk_name}_id"
+            if pk_column_name in df_reshaped.columns:
+                df_reshaped = df_reshaped.set_index(pk_column_name)
+
+        # cast floats and ints where appropriate
+        # this is currently needed because the UI writes into the JSON field through JS
+        # and thus a `10` might be a float, not an int
+        if feature_qs is not None:
+            for feature in feature_qs:
+                if feature.name in df_reshaped.columns:
+                    current_dtype = df_reshaped[feature.name].dtype
+                    if feature.dtype == "int" and not pd.api.types.is_integer_dtype(
+                        current_dtype
+                    ):
+                        df_reshaped[feature.name] = df_reshaped[feature.name].astype(
+                            int
+                        )
+                    elif feature.dtype == "float" and not pd.api.types.is_float_dtype(
+                        current_dtype
+                    ):
+                        df_reshaped[feature.name] = df_reshaped[feature.name].astype(
+                            float
+                        )
         return df_reshaped
 
     @deprecated(new_name="to_dataframe")
@@ -857,7 +957,7 @@ class BasicQuerySet(models.QuerySet):
         include: str | list[str] | None = None,
         features: bool | list[str] | str | None = None,
     ) -> pd.DataFrame:
-        return self.to_dataframe(include, features)
+        return self.to_dataframe(include=include, features=features)
 
     def delete(self, *args, permanent: bool | None = None, **kwargs):
         """Delete all records in the query set.
