@@ -10,6 +10,7 @@ from lamindb_setup.core import deprecated
 from lamindb.base.fields import (
     BooleanField,
     CharField,
+    DateTimeField,
     ForeignKey,
     JSONField,
     TextField,
@@ -21,24 +22,33 @@ from .artifact import Artifact
 from .can_curate import CanCurate
 from .feature import Feature
 from .has_parents import HasParents, _query_ancestors_of_fk, _query_relatives
-from .query_set import QuerySet, SQLRecordList, reorder_subset_columns_in_df
-from .run import Run, TracksRun, TracksUpdates, User, current_run
+from .query_set import (
+    QuerySet,
+    SQLRecordList,
+    encode_lamindb_fields_as_columns,
+    get_default_branch_ids,
+    reorder_subset_columns_in_df,
+)
+from .run import Run, TracksRun, TracksUpdates, User, current_run, current_user_id
 from .sqlrecord import BaseSQLRecord, IsLink, SQLRecord, _get_record_kwargs
 from .transform import Transform
 from .ulabel import ULabel
 
 if TYPE_CHECKING:
+    from datetime import datetime
+
     import pandas as pd
 
-    from .blocks import RunBlock
-    from .project import Project, Reference
+    from ._feature_manager import FeatureManager
+    from .block import RunBlock
+    from .project import Project, RecordProject, RecordReference, Reference
     from .schema import Schema
 
 
 class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
-    """Metadata records for labeling and organizing entities in sheets.
+    """Flexible metadata records for labeling and organizing entities.
 
-    Is useful to manage samples, donors, cells, compounds, sequences.
+    Useful for managing samples, donors, cells, compounds, sequences, and other custom entities.
 
     Args:
         name: `str` A name.
@@ -46,7 +56,7 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         type: `Record | None = None` The type of this record.
         is_type: `bool = False` Whether this record is a type (a record that
             classifies other records).
-        schema: `Schema | None = None` A schema to enforce for a type (optional).
+        schema: `Schema | None = None` A schema defining allowed features for records of this type. Only applicable when `is_type=True`.
         reference: `str | None = None` For instance, an external ID or a URL.
         reference_type: `str | None = None` For instance, `"url"`.
 
@@ -56,54 +66,62 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
 
     Examples:
 
-        Create a record type and then instances of that type::
-
-            sample_type = Record(name="Sample", is_type=True).save()
-            sample1 = Record(name="Sample 1", type=sample_type).save()
-            sample2 = Record(name="Sample 2", type=sample_type).save()
-
-        You can then annotate artifacts and other entities with these records, e.g.::
+        Create a **record** and annotate an :class:`~lamindb.Artifact`::
 
+            sample1 = ln.Record(name="Sample 1").save()
             artifact.records.add(sample1)
 
-        To query artifacts by records::
+        Group several records under a **record type**::
 
-            ln.Artifact.filter(records=sample1).to_dataframe()
+            experiment_type = ln.Record(name="Experiment", is_type=True).save()
+            experiment1 = ln.Record(name="Experiment 1", type=experiment_type).save()
+            experiment2 = ln.Record(name="Experiment 2", type=experiment_type).save()
 
-        Through the UI can assign attributes to records in form of features. The Python API also allows to
-        assign features programmatically, but is currently still low-level::
+        Export all records of that type to dataframe::
 
-            feature = ln.Feature(name="age", type="int").save()
-            sample1.values_record.create(feature=feature, value=42)
-            sample2.values_record.create(feature=feature, value=23)
+            experiment_type.records.to_dataframe()
+            #>              name   ...
+            #>      Experiment 1   ...
+            #>      Experiment 2   ...
 
-        Records can also model flexible ontologies through their parents-children relationships::
+        Add **features** to a record::
 
-            cell_type = Record(name="CellType", is_type=True).save()
-            t_cell = Record(name="T Cell", type=cell_type).save()
-            cd4_t_cell = Record(name="CD4+ T Cell", type=cell_type).save()
+            gc_content = ln.Feature(name="gc_content", dtype=float).save()
+            experiment = ln.Feature(name="experiment", dtype=experiment_type).save()
+            sample1.features.add_values({
+                "gc_content": 0.5,
+                "experiment": "Experiment 1",
+            })
+
+        **Constrain features** by using a :class:`~lamindb.Schema`, creating a **sheet**::
+
+            schema = ln.Schema([gc_content, experiment], name="sample_schema").save()
+            sheet = ln.Record(name="Sample", is_type=True, schema=schema).save()  # add schema to type
+            sample2 = ln.Record(name="Sample 2", type=sheet).save()
+            sample2.features.add_values({"gc_content": 0.6})  # raises ValidationError because experiment is missing
+
+        Query records by features::
+
+            ln.Record.filter(gc_content=0.55)     # exact match
+            ln.Record.filter(gc_content__gt=0.5)  # greater than
+            ln.Record.filter(type=sheet)          # just the record on the sheet
+
+        Model **custom ontologies** through their parents/children attributes::
+
+            cell_type = ln.Record(name="CellType", is_type=True).save()
+            t_cell = ln.Record(name="T Cell", type=cell_type).save()
+            cd4_t_cell = ln.Record(name="CD4+ T Cell", type=cell_type).save()
             t_cell.children.add(cd4_t_cell)
 
-        Often, a label is measured *within* a dataset. For instance, an artifact
-        might characterize 2 species of the Iris flower (`"setosa"` &
-        `"versicolor"`) measured by a `"species"` feature. For such cases, you can use
-        :class:`~lamindb.curators.DataFrameCurator` to automatically parse, validate, and
-        annotate with labels that are contained in `DataFrame` objects.
+        If you work with basic biological entities like cell lines, cell types, tissues,
+        consider building on the public biological ontologies in :mod:`bionty`.
 
-    .. note::
+    .. dropdown:: What is the difference between `Record` and `SQLRecord`?
 
-        If you work with complex entities like cell lines, cell types, tissues,
-        etc., consider using the pre-defined biological registries in
-        :mod:`bionty` to label artifacts & collections.
+        The features of a `Record` are flexible: you can dynamically define features and add features to a record.
+        The fields of a `SQLRecord` are fixed: you need to define them in code and then migrate the underlying database.
 
-        If you work with biological samples, likely, the only sustainable way of
-        tracking metadata, is to create a custom schema module.
-
-    .. note::
-
-        A `Record` has a flexible schema: it can store data for arbitrary features.
-        By contrast, if you want to change the fields of a :class:`~lamindb.models.SQLRecord`, you need to modify the columns of the underlying table in the database.
-        The latter is more efficient for large datasets and you can customize it through modules like the `bionty` or `wetlab` module.
+        You can configure a `SQLRecord` by subclassing it in a custom schema, for example, as done here: `github.com/laminlabs/wetlab <https://github.com/laminlabs/wetlab>`__
 
     """
 
@@ -112,8 +130,11 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         app_label = "lamindb"
         constraints = [
             models.UniqueConstraint(
-                fields=["name", "type", "space"], name="unique_name_type_space"
+                fields=["name", "type", "space"],
+                name="unique_record_name_type_space",
+                condition=~models.Q(branch_id=-1),
             )
+            # also see raw SQL constraints for `is_type` and `type` FK validity in migrations
         ]
 
     _name_field: str = "name"
@@ -135,7 +156,7 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
     Allows to group records by type, e.g., all samples, all donors, all cells, all compounds, all sequences.
     """
     records: Record
-    """Records of this type (can only be non-empty if `is_type` is `True`)."""
+    """If a type (`is_type=True`), records of this `type`."""
     is_type: bool = BooleanField(default=False, db_index=True)
     """Indicates if record is a `type`.
 
@@ -150,33 +171,28 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
     schema: Schema | None = ForeignKey(
         "Schema", CASCADE, null=True, related_name="records"
     )
-    """A schema to enforce for a type (optional).
+    """A schema to enforce for a type.
 
-    This is mostly parallel to the `schema` attribute of `Artifact`.
-
-    If `is_type` is `True`, the schema is used to enforce certain features for each records of this type.
+    This is analogous to the `schema` attribute of an `Artifact`.
+    If `is_type` is `True`, the schema is used to enforce features for each record of this type.
     """
-    # naming convention in analogy to Schema
+    # naming convention in analogy to Schema, but probably better would linked_records in analogy with other
+    # record relationships
     components: Record = models.ManyToManyField(
         "Record", through="RecordRecord", symmetrical=False, related_name="composites"
     )
-    """Record-like components of this record."""
-    composites: Record
-    """Record-like composites of this record."""
-    parents: ULabel = models.ManyToManyField(
+    """Records linked in this record as a value."""
+    composites: Record  # consider renaming to linked_in_records in LaminDB 2
+    """Records linking this record as a value. Is reverse accessor for `components`."""
+    parents: Record = models.ManyToManyField(
         "self", symmetrical=False, related_name="children"
     )
-    """Parent entities of this record.
+    """Ontological parents of this record.
 
-    For advanced use cases, you can build an ontology under a given `type`.
-
-    Say, if you modeled `CellType` as a `Record`, you would introduce a type `CellType` and model the hiearchy of cell types under it.
-    """
-    children: ULabel
-    """Child entities of this record.
-
-    Reverse accessor for parents.
+    You can build an ontology under a given `type`. For example, introduce a type `CellType` and model the hiearchy of cell types under it via `parents` and `children`.
     """
+    children: Record
+    """Ontological children of this record. Is reverse accessor for `parents`."""
     # this is handled manually here because we want to se the related_name attribute
     # (this doesn't happen via inheritance of TracksRun, everything else is the same)
     run: Run | None = ForeignKey(
@@ -194,50 +210,42 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         Artifact, through="ArtifactRecord", related_name="records"
     )
     """Artifacts annotated by this record."""
+    runs: Run = models.ManyToManyField(Run, through="RunRecord", related_name="records")
+    """Runs annotated by this record."""
     projects: Project
     """Projects that annotate this record."""
     references: Reference
     """References that annotate this record."""
-    values_json: RecordJson
-    """JSON values (for lists, dicts, etc.)."""
-    values_record: RecordRecord
-    """Record values with their features."""
-    values_ulabel: RecordULabel
-    """ULabel values with their features."""
-    values_user: RecordUser
-    """User values with their features."""
-    values_run: RecordRun
-    """Run values with their features."""
-    values_artifact: RecordArtifact
-    """Artifact values with their features."""
-    values_reference: Reference
-    """Reference values with their features."""
-    values_project: Project
-    """Project values with their features."""
-    linked_runs: Run = models.ManyToManyField(
-        Run, through="RecordRun", related_name="records"
-    )
+    linked_runs: Run
     """Runs linked in this record as values."""
-    linked_users: User = models.ManyToManyField(
-        User, through="RecordUser", related_name="records"
-    )
-    """Users linked in this record as values."""
-    linked_ulabels: ULabel = models.ManyToManyField(
-        ULabel,
-        through="RecordULabel",
-        related_name="linked_in_records",
-    )
+    linked_ulabels: ULabel
     """ULabels linked in this record as values."""
-    linked_artifacts: Artifact = models.ManyToManyField(
-        Artifact, through="RecordArtifact", related_name="linked_in_records"
-    )
+    linked_artifacts: Artifact
     """Artifacts linked in this record as values."""
     linked_projects: Project
     """Projects linked in this record as values."""
     linked_references: Reference
     """References linked in this record as values."""
+    linked_users: User
+    """Users linked in this record as values."""
     blocks: RunBlock
     """Blocks that annotate this record."""
+    values_json: RecordJson
+    """JSON values `(record_id, feature_id, value)`."""
+    values_record: RecordRecord
+    """Record values with their features `(record_id, feature_id, value_id)`."""
+    values_ulabel: RecordULabel
+    """ULabel values with their features `(record_id, feature_id, value_id)`."""
+    values_user: RecordUser
+    """User values with their features `(record_id, feature_id, value_id)`."""
+    values_run: RecordRun
+    """Run values with their features `(record_id, feature_id, value_id)`."""
+    values_artifact: RecordArtifact
+    """Artifact values with their features `(record_id, feature_id, value_id)`."""
+    values_reference: RecordReference
+    """Reference values with their features `(record_id, feature_id, value_id)`."""
+    values_project: RecordProject
+    """Project values with their features `(record_id, feature_id, value_id)`."""
 
     @overload
     def __init__(
@@ -278,15 +286,17 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         branch_id = kwargs.pop("branch_id", 1)
         space = kwargs.pop("space", None)
         space_id = kwargs.pop("space_id", 1)
-        _skip_validation = kwargs.pop(
-            "_skip_validation", False
-        )  # should not validate records
+        _skip_validation = kwargs.pop("_skip_validation", False)
         _aux = kwargs.pop("_aux", None)
         if len(kwargs) > 0:
             valid_keywords = ", ".join([val[0] for val in _get_record_kwargs(Record)])
             raise FieldValidationError(
                 f"Only {valid_keywords} are valid keyword arguments"
             )
+        if type and not type.is_type:
+            raise ValueError(
+                f"You can only assign a record of `is_type=True` as `type` to another record, but this doesn't have it: {type}"
+            )
         if schema and not is_type:
             logger.important("passing schema, treating as type")
             is_type = True
@@ -306,9 +316,16 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
             _aux=_aux,
         )
 
+    @property
+    def features(self) -> FeatureManager:
+        """Manage annotations with features."""
+        from ._feature_manager import FeatureManager
+
+        return FeatureManager(self)
+
     @property
     def is_form(self) -> bool:
-        """Check if record is a form (a record type with a validating schema)."""
+        """Check if record is a form, i.e., `self.is_type and self.schema is not None`."""
         return self.schema is not None and self.is_type
 
     def query_parents(self) -> QuerySet:
@@ -317,7 +334,7 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         While `.parents` retrieves the direct parents, this method
         retrieves all ancestors of a record type.
         """
-        return _query_relatives([self], "parents", self.__class__)  # type: ignore
+        return _query_relatives([self], "parents")  # type: ignore
 
     def query_children(self) -> QuerySet:
         """Query all children of a record recursively.
@@ -325,7 +342,7 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         While `.children` retrieves the direct children, this method
         retrieves all descendants of a record type.
         """
-        return _query_relatives([self], "children", self.__class__)  # type: ignore
+        return _query_relatives([self], "children")  # type: ignore
 
     def query_records(self) -> QuerySet:
         """Query all records of a type recursively.
@@ -333,7 +350,7 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         While `.records` retrieves the direct children, this method
         retrieves all descendants of a record type.
         """
-        return _query_relatives([self], "records", self.__class__)  # type: ignore
+        return _query_relatives([self], "records")  # type: ignore
 
     def query_types(self) -> SQLRecordList:
         """Query types of a record recursively.
@@ -343,12 +360,37 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
         """
         return _query_ancestors_of_fk(self, "type")  # type: ignore
 
-    def type_to_dataframe(self) -> pd.DataFrame:
-        """Export all instances of this record type to a pandas DataFrame."""
+    def type_to_dataframe(self, recurse: bool = False) -> pd.DataFrame:
+        """Export all instances of this record type to a pandas DataFrame.
+
+        This is almost equivalent to::
+
+            ln.Record.filter(type=sample_type).to_dataframe(include="features")
+
+        `type_to_dataframe()` ensures that the columns are ordered according to the schema of the type and encodes fields like `uid` and `name`.
+
+        Args:
+            recurse: `bool = False` Whether to include records of sub-types recursively.
+        """
         assert self.is_type, "Only types can be exported as dataframes"  # noqa: S101
-        df = self.query_records().to_dataframe(features="queryset")
-        df.columns.values[0] = "__lamindb_record_uid__"
-        df.columns.values[1] = "__lamindb_record_name__"
+
+        branch_ids = get_default_branch_ids()
+        qs = (
+            self.query_records()
+            if recurse
+            else self.records.filter(branch_id__in=branch_ids)
+        )
+        df = qs.to_dataframe(features="queryset", order_by="id")
+        encoded_id = encode_lamindb_fields_as_columns(self.__class__, "id")
+        encoded_uid = encode_lamindb_fields_as_columns(self.__class__, "uid")
+        encoded_name = encode_lamindb_fields_as_columns(self.__class__, "name")
+        # encode the django id, uid and name fields
+        if df.index.name == "id":
+            df.index.name = encoded_id
+        if "uid" in df.columns and encoded_uid not in df.columns:
+            df = df.rename(columns={"uid": encoded_uid})
+        if "name" in df.columns and encoded_name not in df.columns:
+            df = df.rename(columns={"name": encoded_name})
         if self.schema is not None:
             desired_order = self.schema.members.to_list(
                 "name"
@@ -373,8 +415,6 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
             file_suffix = ".csv"
             key = f"sheet_exports/{self.name}{file_suffix}"
         description = f": {self.description}" if self.description is not None else ""
-        format: dict[str, Any] = {"suffix": ".csv"} if key.endswith(".csv") else {}
-        format["index"] = False
         transform, _ = Transform.objects.get_or_create(
             key="__lamindb_record_export__", type="function"
         )
@@ -385,7 +425,7 @@ class Record(SQLRecord, CanCurate, TracksRun, TracksUpdates, HasParents):
             key=key,
             description=f"Export of sheet {self.uid}{description}",
             schema=self.schema,
-            format=format,
+            csv_kwargs={"index": False},
             run=run,
         ).save()
 
@@ -401,7 +441,7 @@ class RecordJson(BaseSQLRecord, IsLink):
         unique_together = ("record", "feature")  # a list is modeled as a list in json
 
 
-class RecordRecord(SQLRecord, IsLink):
+class RecordRecord(BaseSQLRecord, IsLink):
     id: int = models.BigAutoField(primary_key=True)
     record: Record = ForeignKey(
         Record, CASCADE, related_name="values_record"
@@ -440,11 +480,12 @@ class RecordUser(BaseSQLRecord, IsLink):
         unique_together = ("record", "feature", "value")
 
 
+# for storing run-like values in records
 class RecordRun(BaseSQLRecord, IsLink):
     id: int = models.BigAutoField(primary_key=True)
     record: Record = ForeignKey(Record, CASCADE, related_name="values_run")
     feature: Feature = ForeignKey(Feature, PROTECT, related_name="links_recordrun")
-    value: Run = ForeignKey(Run, PROTECT, related_name="links_record")
+    value: Run = ForeignKey(Run, PROTECT, related_name="links_in_record")
 
     class Meta:
         # allows linking several records to a single run for the same feature because we'll likely need this
@@ -452,6 +493,25 @@ class RecordRun(BaseSQLRecord, IsLink):
         unique_together = ("record", "feature", "value")
 
 
+# for annotating runs with records
+class RunRecord(BaseSQLRecord, IsLink):
+    id: int = models.BigAutoField(primary_key=True)
+    run: Run = ForeignKey(Run, CASCADE, related_name="links_record")
+    record: Record = ForeignKey(Record, PROTECT, related_name="links_run")
+    feature: Feature = ForeignKey(Feature, PROTECT, related_name="links_runrecord")
+    created_at: datetime = DateTimeField(
+        editable=False, db_default=models.functions.Now(), db_index=True
+    )
+    created_by: User = ForeignKey(
+        "lamindb.User", PROTECT, default=current_user_id, related_name="+"
+    )
+
+    class Meta:
+        app_label = "lamindb"
+        unique_together = ("run", "record", "feature")
+
+
+# for storing artifact-like values in records
 class RecordArtifact(BaseSQLRecord, IsLink):
     id: int = models.BigAutoField(primary_key=True)
     record: Record = ForeignKey(Record, CASCADE, related_name="values_artifact")
@@ -464,8 +524,8 @@ class RecordArtifact(BaseSQLRecord, IsLink):
         unique_together = ("record", "feature", "value")
 
 
-# like ArtifactULabel, for annotation
-class ArtifactRecord(BaseSQLRecord, IsLink):
+# for annotating artifacts with records
+class ArtifactRecord(BaseSQLRecord, IsLink, TracksRun):
     id: int = models.BigAutoField(primary_key=True)
     artifact: Artifact = ForeignKey(Artifact, CASCADE, related_name="links_record")
     record: Record = ForeignKey(Record, PROTECT, related_name="links_artifact")
