@@ -7,6 +7,7 @@ from django.db.models import (
     CASCADE,
     PROTECT,
 )
+from lamin_utils import logger
 from lamindb_setup import _check_instance_setup
 
 from lamindb.base.fields import (
@@ -129,15 +130,13 @@ class User(BaseSQLRecord, CanCurate):
 
     Every :class:`~lamindb.models.SQLRecord` has a `created_by` field that links to the creating user.
 
-    All data in this registry is synchronized from LaminHub to ensure a universal
-    user identity.
+    This registry is automatically populated with user identities from LaminHub in case the user authenticates.
 
     Examples:
 
-        Query a user by handle:
+        Query a user by handle::
 
-        >>> user = ln.User.get(handle="testuser1")
-        >>> user
+            user = ln.User.get(handle="testuser1")
     """
 
     class Meta:
@@ -153,6 +152,17 @@ class User(BaseSQLRecord, CanCurate):
     """User handle, valid across DB instances (required)."""
     name: str | None = CharField(max_length=150, db_index=True, null=True)
     """Full name (optional)."""  # has to match hub specification, where it's also optional
+    linked_in_records: Record = models.ManyToManyField(
+        "Record", through="RecordUser", related_name="linked_users"
+    )
+    """Records linked in this user."""
+    artifacts: Artifact = models.ManyToManyField(
+        "Artifact",
+        through="ArtifactUser",
+        through_fields=("user", "artifact"),
+        related_name="users",
+    )
+    """Artifacts annotated with this user."""
     created_artifacts: Artifact
     """Artifacts created by user."""
     created_transforms: Transform
@@ -254,6 +264,9 @@ class Run(SQLRecord):
         app_label = "lamindb"
 
     _name_field: str = "started_at"
+    _aux_fields: dict[str, tuple[str, type]] = {
+        "0": ("cli_args", str),
+    }
 
     id: int = models.BigAutoField(primary_key=True)
     """Internal id, valid only in one DB instance."""
@@ -346,6 +359,10 @@ class Run(SQLRecord):
     """Blocks that annotate this run."""
     records: Record
     """Records that annotate this run."""
+    linked_in_records: Record = models.ManyToManyField(
+        "Record", through="RecordRun", related_name="linked_runs"
+    )
+    """This run is linked in these records as a value."""
     _is_consecutive: bool | None = BooleanField(null=True)
     """Indicates whether code was consecutively executed. Is relevant for notebooks."""
     _status_code: int = models.SmallIntegerField(
@@ -432,6 +449,22 @@ class Run(SQLRecord):
         }
         return status_dict.get(self._status_code, "unknown")
 
+    @property
+    def cli_args(self) -> str | None:
+        """CLI arguments if the run was invoked from the command line."""
+        if self._aux is not None and "af" in self._aux and "0" in self._aux["af"]:  # type: ignore
+            return self._aux["af"]["0"]  # type: ignore
+        else:
+            return None
+
+    @cli_args.setter
+    def cli_args(self, value: str) -> None:
+        if not isinstance(value, str) or not (value):
+            logger.warning("did not set empty or non-string cli_args")
+            return
+        self._aux = self._aux or {}
+        self._aux.setdefault("af", {})["0"] = value
+
     @property
     def features(self) -> FeatureManager:
         """Manage annotations with features."""
@@ -439,6 +472,16 @@ class Run(SQLRecord):
 
         return FeatureManager(self)
 
+    def describe(self, return_str: bool = False) -> None | str:
+        """Describe record including relations.
+
+        Args:
+            return_str: Return a string instead of printing.
+        """
+        from ._describe import describe_postgres_sqlite
+
+        return describe_postgres_sqlite(self, return_str=return_str)
+
     @classmethod
     def filter(
         cls,
