@@ -8,7 +8,7 @@ from collections import defaultdict
 from datetime import datetime
 from typing import TYPE_CHECKING
 
-from django.db import transaction
+from django.db import IntegrityError, transaction
 from django.utils.functional import partition
 from lamin_utils import logger
 from lamindb_setup.core.upath import LocalPathClasses, UPath
@@ -21,7 +21,7 @@ from ..core.storage.paths import (
     delete_storage_using_key,
     store_file_or_folder,
 )
-from .sqlrecord import SQLRecord
+from .sqlrecord import SQLRecord, parse_violated_field_from_error_message
 
 if TYPE_CHECKING:
     from collections.abc import Iterable
@@ -105,7 +105,7 @@ def save(
         with transaction.atomic():
             for record in artifacts:
                 # will swtich to True after the successful upload / saving
-                if hasattr(record, "_local_filepath") and getattr(
+                if getattr(record, "_local_filepath", None) is not None and getattr(
                     record, "_to_store", False
                 ):
                     record._is_saved_to_storage_location = False
@@ -139,8 +139,8 @@ def bulk_create(
         total_records = len(records_list)
         model_name = registry.__name__
         if total_records > batch_size:
-            logger.warning(
-                f"Starting bulk_create for {total_records} {model_name} records in batches of {batch_size}"
+            logger.important(
+                f"starting creation of {total_records} {model_name} records in batches of {batch_size}"
             )
 
         # Process records in batches
@@ -151,10 +151,52 @@ def bulk_create(
 
             if total_records > batch_size:
                 logger.info(
-                    f"Processing batch {batch_num}/{total_batches} for {model_name}: {len(batch)} records"
+                    f"processing batch {batch_num}/{total_batches} for {model_name}: {len(batch)} records"
                 )
-            registry.objects.bulk_create(batch, ignore_conflicts=ignore_conflicts)
-            # records[:] = created  # In-place list update; does not seem to be necessary
+            try:
+                registry.objects.bulk_create(batch, ignore_conflicts=ignore_conflicts)
+            # handle unique constraint violations due to non-default branches
+            except IntegrityError as e:
+                error_msg = str(e)
+                if any(
+                    field in error_msg for field in ("root", "ontology_id", "uid")
+                ) and (
+                    "UNIQUE constraint failed" in error_msg
+                    or "duplicate key value violates unique constraint" in error_msg
+                ):
+                    unique_field = parse_violated_field_from_error_message(error_msg)
+                    unique_field_values = [getattr(r, unique_field) for r in batch]
+                    # here we query against non-default branches
+                    pre_existing_values_not_main_branch = (
+                        registry.objects.filter(
+                            **{f"{unique_field}__in": unique_field_values}
+                        )
+                        .exclude(branch_id=1)
+                        .values_list(unique_field, flat=True)
+                    )
+                    # the rest of the records can be saved normally
+                    records_main_branch = [
+                        r
+                        for r in batch
+                        if getattr(r, unique_field)
+                        not in pre_existing_values_not_main_branch
+                    ]
+                    save(records_main_branch)
+                    # now move the pre-existing records to the main branch
+                    if pre_existing_values_not_main_branch.exists():
+                        logger.warning(
+                            f"some {model_name} records with the same {unique_field}s already exist in non-default branches - moving them to the default branch"
+                        )
+                        pre_existing_records_not_main_branch = [
+                            r
+                            for r in batch
+                            if getattr(r, unique_field)
+                            in pre_existing_values_not_main_branch
+                        ]
+                        for record in pre_existing_records_not_main_branch:
+                            record.save()
+                else:
+                    raise e
 
 
 def bulk_update(
@@ -178,7 +220,7 @@ def bulk_update(
         model_name = registry.__name__
         if total_records > batch_size:
             logger.warning(
-                f"Starting bulk_update for {total_records} {model_name} records in batches of {batch_size}"
+                f"starting update for {total_records} {model_name} records in batches of {batch_size}"
             )
 
         field_names = [
@@ -195,7 +237,7 @@ def bulk_update(
 
             if total_records > batch_size:
                 logger.info(
-                    f"Processing batch {batch_num}/{total_batches} for {model_name}: {len(batch)} records"
+                    f"processing batch {batch_num}/{total_batches} for {model_name}: {len(batch)} records"
                 )
             registry.objects.bulk_update(batch, field_names)
 
@@ -211,7 +253,7 @@ def check_and_attempt_upload(
     # kwargs are propagated to .upload_from in the end
     # if Artifact object is either newly instantiated or replace() was called on
     # a local env it will have a _local_filepath and needs to be uploaded
-    if hasattr(artifact, "_local_filepath"):
+    if getattr(artifact, "_local_filepath", None) is not None:
         try:
             storage_path, cache_path = upload_artifact(
                 artifact,
