@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import warnings
 from typing import TYPE_CHECKING, Any, Type, overload
 
 import numpy as np
@@ -8,9 +9,6 @@ from django.db.models import CASCADE, PROTECT, ManyToManyField, Q
 from lamin_utils import logger
 from lamindb_setup.core import deprecated
 from lamindb_setup.core.hashing import HASH_LENGTH, hash_string
-from rich.table import Table
-from rich.text import Text
-from rich.tree import Tree
 
 from lamindb.base import ids
 from lamindb.base.fields import (
@@ -26,7 +24,7 @@ from lamindb.errors import FieldValidationError, InvalidArgument
 from lamindb.models.feature import parse_cat_dtype
 
 from ..errors import ValidationError
-from ._describe import format_rich_tree, highlight_time
+from ._describe import describe_schema, format_rich_tree
 from ._relations import (
     dict_related_model_to_related_name,
     get_related_name,
@@ -104,84 +102,6 @@ def get_features_config(
         return features, configs  # type: ignore
 
 
-def describe_schema(self: Schema) -> Tree:
-    """Create a rich tree visualization of a Schema with its features."""
-    otype = self.otype if hasattr(self, "otype") and self.otype else ""
-    tree = Tree(
-        Text.assemble((self.__class__.__name__, "bold"), (f" {otype}", "bold dim")),
-        guide_style="dim",  # dim the connecting lines
-    )
-
-    tree.add(f".uid = '{self.uid}'")
-    tree.add(f".name = '{self.name}'")
-    if self.description:
-        tree.add(f".description = '{self.description}'")
-    if self.itype:
-        tree.add(f".itype = '{self.itype}'")
-    if self.type:
-        tree.add(f".type = '{self.type}'")
-    tree.add(f".ordered_set = {self.ordered_set}")
-    tree.add(f".maximal_set = {self.maximal_set}")
-    tree.add(f".minimal_set = {self.minimal_set}")
-    if hasattr(self, "created_by") and self.created_by:
-        tree.add(
-            Text.assemble(
-                ".created_by = ",
-                (
-                    self.created_by.handle
-                    if self.created_by.name is None
-                    else f"{self.created_by.handle} ({self.created_by.name})"
-                ),
-            )
-        )
-    if hasattr(self, "created_at") and self.created_at:
-        tree.add(Text.assemble(".created_at = ", highlight_time(str(self.created_at))))
-
-    members = self.members
-
-    # Add features section
-    features = tree.add(
-        Text.assemble(
-            (self.itype, "violet"),
-            (" • ", "dim"),
-            (str(members.count()), "pink1"),
-        )
-    )
-
-    if hasattr(self, "members") and self.members.count() > 0:
-        # create a table for the features
-        feature_table = Table(
-            show_header=True, header_style="dim", box=None, pad_edge=False
-        )
-
-        # Add columns
-        feature_table.add_column("name", style="", no_wrap=True)
-        feature_table.add_column("dtype", style="", no_wrap=True)
-        feature_table.add_column("optional", style="", no_wrap=True)
-        feature_table.add_column("nullable", style="", no_wrap=True)
-        feature_table.add_column("coerce_dtype", style="", no_wrap=True)
-        feature_table.add_column("default_value", style="", no_wrap=True)
-
-        # Add rows for each member
-        optionals = self.optionals.get()
-        for member in self.members:
-            feature_table.add_row(
-                member.name,
-                Text(
-                    str(member.dtype)
-                ),  # needs to be wrapped in Text to display correctly
-                "✓" if optionals.filter(uid=member.uid).exists() else "✗",
-                "✓" if member.nullable else "✗",
-                "✓" if member.coerce_dtype else "✗",
-                str(member.default_value) if member.default_value else "unset",
-            )
-
-        # Add the table to the features branch
-        features.add(feature_table)
-
-    return tree
-
-
 class SchemaOptionals:
     """Manage and access optional features in a schema."""
 
@@ -251,7 +171,8 @@ class SchemaOptionals:
 KNOWN_SCHEMAS = {  # by hash
     "kMi7B_N88uu-YnbTLDU-DA": "0000000000000000",  # valid_features
     "1gocc_TJ1RU2bMwDRK-WUA": "0000000000000001",  # valid_ensembl_gene_ids
-    "UR_ozz2VI2sY8ckXop2RAg": "0000000000000002",  # anndata_ensembl_gene_ids_and_valid_features_in_obs
+    "UR_ozz2VI2sY8ckXop2RAg": "0000000000000002",  # anndata_ensembl_gene_ids_and_valid_features_in_obs (itype='Composite')
+    "aqGWHvyY49W_PHELUMiBMw": "0000000000000002",  # anndata_ensembl_gene_ids_and_valid_features_in_obs (itype=None)
 }
 
 
@@ -260,35 +181,37 @@ class Schema(SQLRecord, CanCurate, TracksRun):
 
     Composite schemas can have multiple slots, e.g., for an `AnnData`, one schema for slot `obs` and another one for `var`.
 
-    To create a schema, one of the following must be passed:
-    - `features`: A list of :class:`~lamindb.Feature` records, e.g., `[Feature(...), Feature(...)]`.
-    - `itype`: A registry field, e.g., `Feature` or `bionty.Gene.ensembl_gene_id`, to constrain feature identifiers to be valid identifiers of the registry.
-    - `slots`: A dictionary mapping slot names to :class:`~lamindb.Schema` objects, e.g., `{"obs": Schema(...), "var": Schema(...), "obsm": Schema(...)}`.
-    - `is_type=True`: To create a schema type, e.g., `ln.Schema(name="ProteinPanel", is_type=True)`.
+    To create a schema, at least one of the following arguments must be passed:
+
+    - `features` - a list of :class:`~lamindb.Feature` records
+    - `itype` - a registry or field to constrain feature identifiers, e.g., `ln.Feature` or `bt.Gene.ensembl_gene_id`
+    - `slots` - a dictionary mapping slot names to :class:`~lamindb.Schema` objects, e.g., for an `AnnData`, `{"obs": Schema(...), "var.T": Schema(...), "obsm": Schema(...)}`
+    - `is_type=True` - a schema type, e.g., `ln.Schema(name="ProteinPanel", is_type=True)`
 
     Args:
         features: `list[SQLRecord] | list[tuple[Feature, dict]] | None = None` Feature
-            records, e.g., `[Feature(...), Feature(...)]` or Features with their config, e.g., `[Feature(...).with_config(optional=True)]`.
-        index: `Feature | None = None` A :class:`~lamindb.Feature` record to validate an index of a `DataFrame` and therefore also, e.g., `AnnData` obs and var indices.
+            records, e.g., `[Feature(...), Feature(...)]` or features with their config, e.g., `[Feature(...).with_config(optional=True)]`.
         slots: `dict[str, Schema] | None = None` A dictionary mapping slot names to :class:`~lamindb.Schema` objects.
-        name: `str | None = None` Name of the Schema.
-        description: `str | None = None` Description of the Schema.
-        flexible: `bool | None = None` Whether to include any feature of the same `itype` in validation
-            and annotation. If no Features are passed, defaults to `True`, otherwise to `False`.
-            This means that if you explicitly pass Features, any additional Features will be disregarded during validation & annotation.
-        type: `Schema | None = None` Type of Schema to group measurements by.
-            Define types like `ln.Schema(name="ProteinPanel", is_type=True)`.
-        is_type: `bool = False` Whether the Schema is a Type.
-        itype: `str | None = None` Feature identifier type to validate against. Must match registry type of provided features.
-        otype: `str | None = None` An object type to define the structure of a composite schema (e.g., DataFrame, AnnData).
-        dtype: `str | None = None` The simple type (e.g., "num", "float", "int").
-            Defaults to `None` for sets of :class:`~lamindb.Feature` records and to `"num"` (e.g., for sets of :class:`~bionty.Gene`) otherwise.
-        minimal_set: `bool = True` Whether all passed Features are required by default.
+        name: `str | None = None` Name of the schema.
+        description: `str | None = None` Description of the schema.
+        itype: `str | None = None` Feature identifier type to validate against, e.g., `ln.Feature` or `bt.Gene.ensembl_gene_id`.
+            Is automatically set to the type of the passed `features`.
+        type: `Schema | None = None` Define schema types like `ln.Schema(name="ProteinPanel", is_type=True)`.
+        is_type: `bool = False` Whether the schema is a type.
+        index: `Feature | None = None` A `Feature` record to validate an index of a `DataFrame` and therefore also, e.g., `AnnData` obs and var indices.
+        flexible: `bool | None = None` Whether to include any feature of the same `itype` during validation
+            & annotation. If `features` is passed, defaults to `False` so that, e.g., additional columns of a `DataFrame` encountered during validation are disregarded.
+            If `features` is not passed, defaults to `True`.
+        otype: `str | None = None` An object type to define the structure of a composite schema, e.g., `"DataFrame"`, `"AnnData"`.
+        dtype: `str | None = None` A `dtype` to assume for all features in the schema (e.g., "num", float, int).
+            Defaults to `None` if `itype` is `Feature`. Otherwise to `"num"`, e.g., if `itype` is `bt.Gene.ensembl_gene_id`.
+        minimal_set: `bool = True` Whether all passed features are required by default.
             See :attr:`~lamindb.Schema.optionals` for more-fine-grained control.
-        maximal_set: `bool = False` Whether additional Features are allowed.
-        ordered_set: `bool = False` Whether Features are required to be ordered.
+        maximal_set: `bool = False` Whether additional features are allowed.
+        ordered_set: `bool = False` Whether features are required to be ordered.
         coerce_dtype: `bool = False` When True, attempts to coerce values to the specified dtype
             during validation, see :attr:`~lamindb.Schema.coerce_dtype`.
+        n: `int | None = None` A manual way of specifying the number of features in the schema. Is inferred from `features` if passed.
 
     See Also:
         :meth:`~lamindb.Artifact.from_dataframe`
@@ -317,15 +240,15 @@ class Schema(SQLRecord, CanCurate, TracksRun):
 
             # a schema that constrains feature identifiers to be a valid ensembl gene ids or feature names
             schema = ln.Schema(itype=bt.Gene.ensembl_gene_id)
-            schema = ln.Schema(itype=ln.Feature)  # is equivalent to itype=ln.Feature.name
+            schema = ln.Schema(itype=ln.Feature)  # uses Feature.name as identifier type
 
             # a schema that requires a single feature but also validates & annotates any additional features with valid feature names
             schema = ln.Schema(
                 features=[
                     ln.Feature(name="required_feature", dtype=str).save(),
                 ],
-                itype=ln.Schema(itype=ln.Feature),
-                flexible=True,
+                itype=ln.Feature,
+                flexible=True,  # also validate & annotate additional features if they are present
             ).save()
 
         Passing options to the `Schema` constructor::
@@ -346,7 +269,7 @@ class Schema(SQLRecord, CanCurate, TracksRun):
                 ],
             ).save()
 
-        Alternative constructors (:meth:`~lamindb.Schema.from_values`, :meth:`~lamindb.Schema.from_df`)::
+        Alternative constructors (:meth:`~lamindb.Schema.from_values`, :meth:`~lamindb.Schema.from_dataframe`)::
 
             # parse & validate identifier values
             schema = ln.Schema.from_values(
@@ -363,6 +286,14 @@ class Schema(SQLRecord, CanCurate, TracksRun):
     class Meta(SQLRecord.Meta, TracksRun.Meta, TracksUpdates.Meta):
         abstract = False
         app_label = "lamindb"
+        constraints = [
+            models.UniqueConstraint(
+                fields=["name", "type", "space"],
+                name="unique_schema_name_type_space",
+                condition=~models.Q(branch_id=-1),
+            )
+            # also see raw SQL constraints for `is_type` and `type` FK validity in migrations
+        ]
 
     _name_field: str = "name"
     _aux_fields: dict[str, tuple[str, type]] = {
@@ -485,19 +416,20 @@ class Schema(SQLRecord, CanCurate, TracksRun):
         | SQLRecordList
         | list[tuple[Feature, dict]]
         | None = None,
-        index: Feature | None = None,
+        *,
         slots: dict[str, Schema] | None = None,
         name: str | None = None,
         description: str | None = None,
         itype: str | Registry | FieldAttr | None = None,
-        flexible: bool | None = None,
         type: Schema | None = None,
         is_type: bool = False,
+        index: Feature | None = None,
+        flexible: bool | None = None,
         otype: str | None = None,
         dtype: str | Type[int | float | str] | None = None,  # noqa
-        ordered_set: bool = False,
         minimal_set: bool = True,
         maximal_set: bool = False,
+        ordered_set: bool = False,
         coerce_dtype: bool = False,
         n: int | None = None,
     ): ...
@@ -638,30 +570,25 @@ class Schema(SQLRecord, CanCurate, TracksRun):
     ) -> tuple[list[Feature], dict[str, Any], list[Feature], Registry, bool]:
         optional_features = []
         features_registry: Registry = None
-
         if itype is not None:
             if itype != "Composite":
                 itype = serialize_dtype(itype, is_itype=True)
-
+            else:
+                warnings.warn(
+                    "please do not pass the deprecated itype='Composite'", stacklevel=2
+                )
         if index is not None:
             if not isinstance(index, Feature):
                 raise TypeError("index must be a Feature")
             features.insert(0, index)
-
-        if slots:
-            itype = "Composite"
-            if otype is None:
-                raise InvalidArgument("Please pass otype != None for composite schemas")
-
         if features:
             features, configs = get_features_config(features)
             features_registry = validate_features(features)
-            if itype != "Composite":
-                itype_compare = features_registry.__get_name_with_module__()
-                if itype is not None:
-                    assert itype.startswith(itype_compare), str(itype_compare)  # noqa: S101
-                else:
-                    itype = itype_compare
+            itype_compare = features_registry.__get_name_with_module__()
+            if itype is not None:
+                assert itype.startswith(itype_compare), str(itype_compare)  # noqa: S101
+            else:
+                itype = itype_compare
             if n_features is not None:
                 if n_features != len(features):
                     logger.important(f"updating to n {len(features)} features")
@@ -680,11 +607,12 @@ class Schema(SQLRecord, CanCurate, TracksRun):
             dtype = None if itype is not None and itype == "Feature" else NUMBER_TYPE
         else:
             dtype = get_type_str(dtype)
+        if slots:
+            if otype is None:
+                raise InvalidArgument("Please pass otype != None for composite schemas")
         flexible_default = n_features < 0
-
         if flexible is None:
             flexible = flexible_default
-
         if itype is not None and not isinstance(itype, str):
             itype_str = serialize_dtype(itype, is_itype=True)
         else:
@@ -1161,17 +1089,15 @@ class Schema(SQLRecord, CanCurate, TracksRun):
 
                 # access slots
                 anndata_schema.slots
-                # {'obs': <Schema: obs_schema>, 'var': <Schema: var_schema>}
+                #> {'obs': <Schema: obs_schema>, 'var': <Schema: var_schema>}
         """
         if hasattr(self, "_slots"):
             return self._slots
-        if self.itype == "Composite":
-            self._slots = {
-                link.slot: link.component
-                for link in self.components.through.filter(composite_id=self.id)
-            }
-            return self._slots
-        return {}
+        self._slots = {
+            link.slot: link.component
+            for link in self.components.through.filter(composite_id=self.id)
+        }
+        return self._slots
 
     @property
     def optionals(self) -> SchemaOptionals:
@@ -1215,23 +1141,12 @@ class Schema(SQLRecord, CanCurate, TracksRun):
         """Describe schema."""
         if self.pk is None:
             raise ValueError("Schema must be saved before describing")
-
-        message = str(self)
-        # display slots for composite schemas
-        if self.itype == "Composite":
-            message + "\nslots:"
-            for slot, schema in self.slots.items():
-                message += f"\n    {slot}: " + str(schema)
-        else:
-            tree = describe_schema(self)
-            return format_rich_tree(
-                tree, fallback="no linked features", return_str=return_str
-            )
-        if return_str:
-            return message
-        else:
-            print(message)
-            return None
+        tree = describe_schema(self)
+        for slot, schema in self.slots.items():
+            tree.add(describe_schema(schema, slot=slot))
+        return format_rich_tree(
+            tree, fallback="no linked features", return_str=return_str
+        )
 
 
 def get_type_str(dtype: str | None) -> str | None:
