@@ -72,7 +72,7 @@ if TYPE_CHECKING:
     import pandas as pd
 
     from .artifact import Artifact
-    from .blocks import Block
+    from .block import BranchBlock, SpaceBlock
     from .run import Run, User
     from .transform import Transform
 
@@ -135,6 +135,25 @@ def unique_constraint_error_in_error_message(error_msg: str) -> bool:
     )
 
 
+def parse_violated_field_from_error_message(error_msg: str) -> str | None:
+    # Even if the model has multiple fields with unique=True,
+    # Django will only raise an IntegrityError for one field at a time
+    # - whichever constraint is violated first during the database insert/update operation.
+    if unique_constraint_error_in_error_message(error_msg):
+        if "UNIQUE constraint failed" in error_msg:  # sqlite
+            constraint_field = (
+                error_msg.removeprefix("UNIQUE constraint failed: ")
+                .split(", ")[0]
+                .split(".")[-1]
+            )
+        else:  # postgres
+            constraint_field = (
+                error_msg.split('"')[1].removesuffix("_key").split("_")[-1]
+            )
+        return constraint_field
+    return None
+
+
 FieldAttr.__repr__ = deferred_attribute__repr__  # type: ignore
 
 
@@ -167,12 +186,18 @@ def is_approx_pascal_case(s):
 
 
 def init_self_from_db(self: SQLRecord, existing_record: SQLRecord):
+    from .run import current_run
+
     new_args = [
         getattr(existing_record, field.attname) for field in self._meta.concrete_fields
     ]
     super(self.__class__, self).__init__(*new_args)
     self._state.adding = False  # mimic from_db
     self._state.db = "default"
+    # if run was not set on the existing record, set it to the current_run
+    if hasattr(self, "run_id") and self.run_id is None and current_run() is not None:
+        logger.warning(f"run was not set on {self}, setting to current run")
+        self.run = current_run()
 
 
 def update_attributes(record: SQLRecord, attributes: dict[str, str]):
@@ -471,16 +496,13 @@ class Registry(ModelBase):
             queries: One or multiple `Q` objects.
             expressions: Fields and values passed as Django query expressions.
 
-        Returns:
-            A :class:`~lamindb.models.QuerySet`.
-
         See Also:
             - Guide: :doc:`docs:registries`
             - Django documentation: `Queries <https://docs.djangoproject.com/en/stable/topics/db/queries/>`__
 
         Examples:
-            >>> ln.ULabel(name="my label").save()
-            >>> ln.ULabel.filter(name__startswith="my").to_dataframe()
+            >>> ln.Project(name="my label").save()
+            >>> ln.Project.filter(name__startswith="my").to_dataframe()
         """
         from .query_set import QuerySet
 
@@ -502,18 +524,18 @@ class Registry(ModelBase):
             expressions: Fields and values passed as Django query expressions.
 
         Raises:
-            :exc:`docs:lamindb.errors.DoesNotExist`: In case no matching record is found.
+            :exc:`lamindb.errors.DoesNotExist`: In case no matching record is found.
 
         See Also:
-            - Guide: :doc:`docs:registries`
+            - Guide: :doc:`registries`
             - Django documentation: `Queries <https://docs.djangoproject.com/en/stable/topics/db/queries/>`__
 
         Examples:
 
             ::
 
-                ulabel = ln.ULabel.get("FvtpPJLJ")
-                ulabel = ln.ULabel.get(name="my-label")
+                record = ln.Record.get("FvtpPJLJ")
+                record = ln.Record.get(name="my-label")
         """
         from .query_set import QuerySet
 
@@ -521,56 +543,62 @@ class Registry(ModelBase):
 
     def to_dataframe(
         cls,
+        *,
         include: str | list[str] | None = None,
-        features: bool | list[str] | str = False,
-        limit: int = 100,
+        features: str | list[str] | None = None,
+        limit: int | None = 100,
+        order_by: str | None = "-id",
     ) -> pd.DataFrame:
-        """Convert to `pd.DataFrame`.
+        """Evaluate and convert to `pd.DataFrame`.
 
-        By default, shows all direct fields, except `updated_at`.
+        By default, maps simple fields and foreign keys onto `DataFrame` columns.
 
-        Use arguments `include` or `feature` to include other data.
+        Guide: :doc:`docs:registries`
 
         Args:
-            include: Related fields to include as columns. Takes strings of
-                form `"ulabels__name"`, `"cell_types__name"`, etc. or a list
-                of such strings.
-            features: If a list of feature names, filters
-                :class:`~lamindb.Feature` down to these features.
-                If `True`, prints all features with dtypes in the core schema module.
-                If `"queryset"`, infers the features used within the set of artifacts or records.
-                Only available for `Artifact` and `Record`.
-            limit: Maximum number of rows to display from a Pandas DataFrame.
-                Defaults to 100 to reduce database load.
+            include: Related data to include as columns. Takes strings of
+                form `"records__name"`, `"cell_types__name"`, etc. or a list
+                of such strings. For `Artifact`, `Record`, and `Run`, can also pass `"features"`
+                to include features with data types pointing to entities in the core schema.
+                If `"privates"`, includes private fields (fields starting with `_`).
+            features: Configure the features to include. Can be a feature name or a list of such names.
+                If `"queryset"`, infers the features used within the current queryset.
+                Only available for `Artifact`, `Record`, and `Run`.
+            limit: Maximum number of rows to display. If `None`, includes all results.
+            order_by: Field name to order the records by. Prefix with '-' for descending order.
+                Defaults to '-id' to get the most recent records. This argument is ignored
+                if the queryset is already ordered or if the specified field does not exist.
 
         Examples:
 
-            Include the name of the creator in the `DataFrame`:
+            Include the name of the creator::
 
-            >>> ln.ULabel.to_dataframe(include="created_by__name"])
+                ln.Record.to_dataframe(include="created_by__name"])
 
-            Include display of features for `Artifact`:
+            Include features::
 
-            >>> df = ln.Artifact.to_dataframe(features=True)
-            >>> ln.view(df)  # visualize with type annotations
+                ln.Artifact.to_dataframe(include="features")
 
-            Only include select features:
+            Include selected features::
 
-            >>> df = ln.Artifact.to_dataframe(features=["cell_type_by_expert", "cell_type_by_model"])
+                ln.Artifact.to_dataframe(features=["cell_type_by_expert", "cell_type_by_model"])
         """
-        query_set = cls.filter()
-        if hasattr(cls, "updated_at"):
-            query_set = query_set.order_by("-updated_at")
-        return query_set[:limit].to_dataframe(include=include, features=features)
+        return cls.filter().to_dataframe(
+            include=include, features=features, order_by=order_by, limit=limit
+        )
 
     @deprecated(new_name="to_dataframe")
     def df(
         cls,
+        *,
         include: str | list[str] | None = None,
-        features: bool | list[str] | str = False,
-        limit: int = 100,
+        features: str | list[str] | None = None,
+        limit: int | None = 100,
+        order_by: str | None = "-id",
     ) -> pd.DataFrame:
-        return cls.to_dataframe(include, features, limit)
+        return cls.to_dataframe(
+            include=include, features=features, limit=limit, order_by=order_by
+        )
 
     @doc_args(_search.__doc__)
     def search(
@@ -590,31 +618,38 @@ class Registry(ModelBase):
             case_sensitive=case_sensitive,
         )
 
+    @deprecated(new_name="connect")
     def using(
         cls,
         instance: str | None,
     ) -> QuerySet:
-        """Use a non-default LaminDB instance.
+        return cls.connect(
+            instance=instance,
+        )
+
+    def connect(
+        cls,
+        instance: str | None,
+    ) -> QuerySet:
+        """Query a non-default LaminDB instance.
 
         Args:
             instance: An instance identifier of form "account_handle/instance_name".
 
         Examples:
-            >>> ln.ULabel.using("account_handle/instance_name").search("ULabel7", field="name")
-                        uid    score
-            name
-            ULabel7  g7Hk9b2v  100.0
-            ULabel5  t4Jm6s0q   75.0
-            ULabel6  r2Xw8p1z   75.0
+
+            ::
+
+                ln.Record.connect("account_handle/instance_name").search("label7", field="name")
         """
         from .query_set import QuerySet
 
-        # connection already established
-        if instance in connections:
-            return QuerySet(model=cls, using=instance)
         # we're in the default instance
         if instance is None or instance == "default":
             return QuerySet(model=cls, using=None)
+        # connection already established
+        if instance in connections:
+            return QuerySet(model=cls, using=instance)
 
         owner, name = get_owner_name_from_identifier(instance)
         current_instance_owner_name: list[str] = setup_settings.instance.slug.split("/")
@@ -898,64 +933,54 @@ class BaseSQLRecord(models.Model, metaclass=Registry):
                     super().save(*args, **kwargs)
             except (IntegrityError, ProgrammingError) as e:
                 error_msg = str(e)
-                # two possible error messages for hash duplication
-                # "duplicate key value violates unique constraint"
-                # "UNIQUE constraint failed"
+                # error for hash/uid duplication
                 if (
                     self.__class__.__name__ in {"Transform", "Artifact", "Collection"}
                     and isinstance(e, IntegrityError)
                     and "hash" in error_msg
                     and unique_constraint_error_in_error_message(error_msg)
                 ):
-                    pre_existing_record = self.__class__.get(hash=self.hash)
+                    # we also need to include the key here because hash can be the same across keys
+                    query_fields = {"hash": self.hash, "key": self.key}
+                    if self.__class__.__name__ == "Artifact":
+                        # in case of artifact, also storage is needed
+                        query_fields["storage"] = self.storage
+                    # the get here is Django's get and not aware of the trash or other branches
+                    # but generally we bypass branch_id in queries for hash also in LaminDB's get()
+                    pre_existing_record = self.__class__.get(**query_fields)
+                    from_trash = (
+                        "from trash" if pre_existing_record.branch_id == -1 else ""
+                    )
+                    pre_existing_record.branch_id = 1  # move to default branch
                     logger.warning(
-                        f"returning {self.__class__.__name__.lower()} with same hash: {pre_existing_record}"
+                        f"returning {self.__class__.__name__.lower()} {from_trash} with same hash & key: {pre_existing_record}"
                     )
                     init_self_from_db(self, pre_existing_record)
-                elif (
-                    self.__class__.__name__ == "Storage"
-                    and isinstance(e, IntegrityError)
-                    and ("root" in error_msg or "uid" in error_msg)
-                    and unique_constraint_error_in_error_message(error_msg)
-                ):
-                    # even if uid was in the error message, we can retrieve based on
-                    # the root because it's going to be the same root
-                    pre_existing_record = self.__class__.get(root=self.root)
-                    init_self_from_db(self, pre_existing_record)
                 elif (
                     isinstance(e, IntegrityError)
-                    and ("ontology_id" in error_msg or "uid" in error_msg)
+                    # for Storageeven if uid was in the error message, we can retrieve based on
+                    # the root because it's going to be the same root
+                    and any(
+                        field in error_msg for field in ("root", "ontology_id", "uid")
+                    )
                     and (
                         "UNIQUE constraint failed" in error_msg
                         or "duplicate key value violates unique constraint" in error_msg
                     )
                 ):
-                    if "UNIQUE constraint failed" in error_msg:  # sqlite
-                        constraint_fields = [
-                            f.split(".")[-1]
-                            for f in error_msg.removeprefix(
-                                "UNIQUE constraint failed: "
-                            ).split(", ")
-                        ]
-                    else:  # postgres
-                        constraint_fields = [
-                            error_msg.split('"')[1]
-                            .split('"')[0]
-                            .removesuffix("_key")
-                            .split("_")[-1]  # field name
-                        ]
+                    unique_field = parse_violated_field_from_error_message(error_msg)
                     # here we query against the all branches with .objects
                     pre_existing_record = self.__class__.objects.get(
-                        **{f: getattr(self, f) for f in constraint_fields}
+                        **{unique_field: getattr(self, unique_field)}
                     )
+                    # if the existing record is in the default branch, we just return it
                     if pre_existing_record.branch_id == 1:
                         logger.warning(
-                            f"returning {self.__class__.__name__} record with same {', '.join(constraint_fields)}: '{', '.join([str(getattr(self, f)) for f in constraint_fields])}'"
+                            f"returning {self.__class__.__name__} record with same {unique_field}: '{getattr(self, unique_field)}'"
                         )
+                    # if the existing record is in a different branch, we move it to the default branch and update its fields
                     else:
                         # modifies the fields of the existing record with new values of self
-                        # TODO: parents should be properly dealt with
-                        self._parents: list = []
                         field_names = [i.name for i in self.__class__._meta.fields]
                         update_attributes(
                             pre_existing_record,
@@ -1087,7 +1112,7 @@ class Space(BaseSQLRecord):
         "User", CASCADE, default=None, related_name="+", null=True
     )
     """Creator of space."""
-    blocks: Block
+    blocks: SpaceBlock
     """Blocks that annotate this space."""
 
     @overload
@@ -1179,7 +1204,7 @@ class Branch(BaseSQLRecord):
         "User", CASCADE, default=None, related_name="+", null=True
     )
     """Creator of branch."""
-    blocks: Block
+    blocks: BranchBlock
     """Blocks that annotate this branch."""
 
     @overload
@@ -1288,7 +1313,7 @@ class SQLRecord(BaseSQLRecord, metaclass=Registry):
                 raise IntegrityError(
                     "Cannot simply delete artifacts outside of this instance's managed storage locations."
                     "\n(1) If you only want to delete the metadata record in this instance, pass `storage=False`"
-                    f"\n(2) If you want to delete the artifact in storage, please load the managing lamindb instance (uid={self.storage.instance_uid})."
+                    f"\n(2) If you want to delete the artifact in storage, please connect to the writing lamindb instance (uid={self.storage.instance_uid})."
                     f"\nThese are all managed storage locations of this instance:\n{Storage.filter(instance_uid=isettings.uid).to_dataframe()}"
                 )
 
@@ -1539,7 +1564,7 @@ def get_transfer_run(record) -> Run:
         ln_setup.settings.cache_dir / f"instance--{owner}--{name}--uid.txt"
     )
     if not cache_using_filepath.exists():
-        raise SystemExit("Need to call .using() before")
+        raise SystemExit("Need to call .connect() before")
     instance_uid = cache_using_filepath.read_text().split("\n")[0]
     key = f"__lamindb_transfer__/{instance_uid}"
     uid = instance_uid + "0000"
@@ -1714,6 +1739,8 @@ def check_key_change(record: Union[Artifact, Transform]):
 
     if not isinstance(record, Artifact) or not hasattr(record, "_old_key"):
         return
+    if hasattr(record, "_skip_key_change_check") and record._skip_key_change_check:
+        return
     if record._old_suffix != record.suffix:  # type: ignore
         raise InvalidArgument(
             f"Changing the `.suffix` of an artifact is not allowed! You tried to change it from '{record._old_suffix}' to '{record.suffix}'."  # type: ignore
@@ -1746,12 +1773,11 @@ def check_key_change(record: Union[Artifact, Transform]):
             )
 
 
-def format_field_value(value: datetime | str | Any) -> Any:
+def format_field_value(value: datetime | str | Any, none: str = "None") -> str:
     from datetime import datetime
 
     if isinstance(value, datetime):
         return value.strftime("%Y-%m-%d %H:%M:%S %Z")
-
     if isinstance(value, str):
         try:
             value = datetime.fromisoformat(value)
@@ -1759,8 +1785,9 @@ def format_field_value(value: datetime | str | Any) -> Any:
         except ValueError:
             pass
         return f"'{value}'"
-    else:
-        return value
+    if value is None:
+        return none
+    return str(value)
 
 
 class SQLRecordInfo:
@@ -1858,6 +1885,21 @@ class SQLRecordInfo:
             class_specific_relational_fields + filtered_non_class_specific
         )
 
+        # For Record class, move linked_in fields to the end
+        if self.registry.__name__ == "Record":
+            regular_fields = [
+                f
+                for f in ordered_relational_fields
+                if not f.name.startswith(("linked_", "values_"))
+            ]
+            linked_fields = [
+                f for f in ordered_relational_fields if f.name.startswith("linked_")
+            ]
+            values_fields = [
+                f for f in ordered_relational_fields if f.name.startswith("values_")
+            ]
+            ordered_relational_fields = regular_fields + linked_fields + values_fields
+
         core_module_fields = []
         external_modules_fields = []
         for field in ordered_relational_fields:
