@@ -30,6 +30,7 @@ if TYPE_CHECKING:
     from .collection import Collection
     from .project import Project
     from .query_set import SQLRecordList
+    from .record import Record
 
 
 class ULabel(SQLRecord, HasParents, CanCurate, TracksRun, TracksUpdates):
@@ -73,6 +74,14 @@ class ULabel(SQLRecord, HasParents, CanCurate, TracksRun, TracksUpdates):
     class Meta(SQLRecord.Meta, TracksRun.Meta, TracksUpdates.Meta):
         abstract = False
         app_label = "lamindb"
+        constraints = [
+            models.UniqueConstraint(
+                fields=["name", "type", "space"],
+                name="unique_ulabel_name_type_space",
+                condition=~models.Q(branch_id=-1),
+            )
+            # also see raw SQL constraints for `is_type` and `type` FK validity in migrations
+        ]
 
     _name_field: str = "name"
 
@@ -117,15 +126,23 @@ class ULabel(SQLRecord, HasParents, CanCurate, TracksRun, TracksUpdates):
     Reverse accessor for parents.
     """
     transforms: Transform
-    """Linked transforms."""
+    """The transforms annotated by this ulabel."""
     runs: Run
-    """Linked runs."""
-    artifacts: Artifact
-    """Linked artifacts."""
+    """The runs annotated by this ulabel."""
+    artifacts: Artifact = models.ManyToManyField(
+        "Artifact", through="ArtifactULabel", related_name="ulabels"
+    )
+    """The artifacts annotated by this ulabel."""
     collections: Collection
-    """Linked collections."""
+    """The collections annotated by this ulabel."""
     projects: Project
-    """Linked projects."""
+    """The projects annotated by this ulabel."""
+    linked_in_records: Record = models.ManyToManyField(
+        "Record",
+        through="RecordULabel",
+        related_name="linked_ulabels",
+    )
+    """Records linking this ulabel as a value."""
 
     @overload
     def __init__(
