@@ -56,7 +56,7 @@ def ccaplog(caplog):
 
 
 @pytest.fixture(
-    scope="module",
+    scope="function",
     params=[
         # tuple of is_in_registered_storage, path, suffix, hash of test_dir
         (True, "./default_storage_unit_core/", ".csv", "iGtHiFEBV3r1_TFovdQCgw"),
@@ -68,8 +68,6 @@ def ccaplog(caplog):
     ],
 )
 def get_test_filepaths(request):  # -> Tuple[bool, Path, Path, Path, str]
-    import lamindb as ln
-
     is_in_registered_storage: bool = request.param[0]
     root_dir: Path = Path(request.param[1])
     suffix: str = request.param[2]
@@ -107,7 +105,7 @@ def get_test_filepaths(request):  # -> Tuple[bool, Path, Path, Path, str]
 
 
 @pytest.fixture(scope="session")
-def df():
+def example_dataframe():
     return pd.DataFrame({"feat1": [1, 2], "feat2": [3, 4]})
 
 
