@@ -15,7 +15,7 @@ from lamindb.models.query_set import DoesNotExist
 
 
 # please also see the test_curate_df.py tests
-def test_df():
+def test_to_dataframe():
     project_label = ln.Record(name="project").save()
     project_names = [f"Project {i}" for i in range(3)]
     labels = ln.Record.from_values(project_names, create=True).save()
@@ -56,13 +56,13 @@ def test_df():
 
     # raise error for non many-to-many
     df = ln.Record.filter(name="Project 0").to_dataframe(include="created_by__name")
-    assert df["created_by__name"].iloc[0] == "Test User1"
+    assert df["created_by__name"].iloc[0] == ln.setup.settings.user.name
 
     # do not return fields with no data in the registry
     # does not make sense in Alex's opinion
     # too much magic; got removed in https://github.com/laminlabs/lamindb/pull/2238
     # df = (
-    #     ln.Artifact.using("laminlabs/cellxgene")
+    #     ln.Artifact.connect("laminlabs/cellxgene")
     #     .filter(suffix=".h5ad")
     #     .to_dataframe(include=["tissues__name", "pathways__name"])
     # )
@@ -81,7 +81,13 @@ def test_df():
 
     # call it from a non-select-derived queryset
     qs = ln.User.objects.all()
-    assert qs.to_dataframe().iloc[0]["handle"] == "testuser1"
+    assert qs.to_dataframe().iloc[0]["handle"] == ln.setup.settings.user.handle
+
+
+def test_complex_df_with_features():
+    # should not fail
+    ln.Artifact.connect("laminlabs/lamindata").to_dataframe(include="features")
+    ln.Artifact.connect("laminlabs/lamindata").to_dataframe(features="queryset")
 
 
 def test_one_first():
@@ -302,3 +308,22 @@ def test_queryset_soft_delete_error():
 
     with pytest.raises(ValueError):
         ln.Branch.filter().delete(permanent=False)
+
+
+def test_encode_lamindb_fields_as_columns():
+    from lamindb.models.query_set import encode_lamindb_fields_as_columns
+
+    assert encode_lamindb_fields_as_columns(
+        ln.Artifact, ["uid", "name", "created_by", "key", "tissues"]
+    ) == {
+        "uid": "__lamindb_artifact_uid__",
+        "created_by": "__lamindb_artifact_created_by__",
+        "key": "__lamindb_artifact_key__",
+    }
+    assert encode_lamindb_fields_as_columns(
+        ln.Record, ["uid", "name", "created_by", "key", "tissues"]
+    ) == {
+        "uid": "__lamindb_record_uid__",
+        "name": "__lamindb_record_name__",
+        "created_by": "__lamindb_record_created_by__",
+    }
