@@ -56,8 +56,6 @@ def test_schema_from_values():
         schema = ln.Schema.from_values(
             ["weird_name"], field=ln.Feature.name, dtype="float"
         )
-    with pytest.raises(ValidationError):
-        ln.Schema.from_values(["name"], field=ln.Feature.name, dtype="float")
 
 
 def test_schema_from_records(df):
@@ -497,22 +495,21 @@ def test_schemas_anndata():
     )
     assert schema._list_for_hashing == [
         "a=num",
-        "b=Composite",
         "c=True",
         "d=False",
         "e=False",
         "l=GPZ-TzvKRhdC1PQAhlFiow",
     ]
     assert schema.name == "anndata_ensembl_gene_ids_and_valid_features_in_obs"
-    assert schema.itype == "Composite"
-    assert schema.hash == "UR_ozz2VI2sY8ckXop2RAg"
+    assert schema.itype is None
+    assert schema.hash == "aqGWHvyY49W_PHELUMiBMw"
 
     # test the convenience function
     schema = ln.examples.schemas.anndata_ensembl_gene_ids_and_valid_features_in_obs()
     assert schema.uid == "0000000000000002"
     assert schema.name == "anndata_ensembl_gene_ids_and_valid_features_in_obs"
-    assert schema.itype == "Composite"
-    assert schema.hash == "UR_ozz2VI2sY8ckXop2RAg"
+    assert schema.itype is None
+    assert schema.hash == "aqGWHvyY49W_PHELUMiBMw"
     varT_schema = schema.slots["var.T"]
     assert varT_schema.uid == "0000000000000001"
     assert varT_schema.name == "valid_ensembl_gene_ids"
