@@ -191,8 +191,6 @@ def spatialdata_blobs_schema():
 def test_dataframe_curator(mini_immuno_schema: ln.Schema):
     """Test DataFrame curator implementation."""
 
-    ln.settings.verbosity = "info"
-
     # invalid simple dtype (float)
     feature_to_fail = ln.Feature(name="treatment_time_h", dtype=float).save()
     schema = ln.Schema(
@@ -250,13 +248,13 @@ def test_dataframe_curator(mini_immuno_schema: ln.Schema):
     assert (
         artifact.features.describe(return_str=True)
         == """\
-Artifact .parquet · DataFrame · dataset
+Artifact: examples/dataset1.parquet (0000)
 └── Dataset features
-    └── columns • 5         [Feature]
-        cell_type_by_expe…  cat[bionty.CellType]    B cell, CD8-positive, alpha…
-        cell_type_by_model  cat[bionty.CellType]    B cell, T cell
-        perturbation        cat[ULabel[Perturbati…  DMSO, IFNG
-        sample_label        cat[ULabel]             sample1, sample2, sample3
+    └── columns (5)
+        cell_type_by_expe…  bionty.CellType         B cell, CD8-positive, alpha…
+        cell_type_by_model  bionty.CellType         B cell, T cell
+        perturbation        ULabel[Perturbation]    DMSO, IFNG
+        sample_label        ULabel                  sample1, sample2, sample3
         sample_note         str"""
     )
     assert set(artifact.features.get_values()["sample_label"]) == {
@@ -381,7 +379,11 @@ def test_dataframe_attrs_validation(study_metadata_schema, include_attrs_slot):
             otype="DataFrame",
         ).save()
 
-    bad_schema = ln.Schema(slots={"doesnotexist": schema}, otype="DataFrame").save()
+    bad_schema = ln.Schema(
+        features=[perturbation_feature],
+        slots={"doesnotexist": schema},
+        otype="DataFrame",
+    ).save()
 
     with pytest.raises(ValueError) as e:
         curator = ln.curators.DataFrameCurator(df, schema=bad_schema)
@@ -871,22 +873,21 @@ def test_spatialdata_curator(
         "tables:table:var.T",
     }
     assert artifact.features.get_values()["disease"] == "Alzheimer disease"
-
     assert (
         artifact.features.describe(return_str=True)
-        == """Artifact .zarr · SpatialData · dataset
+        == """Artifact: examples/spatialdata1.zarr (0000)
 └── Dataset features
-    ├── attrs:bio • 2       [Feature]
-    │   developmental_sta…  cat[bionty.Developmen…  adult stage
-    │   disease             cat[bionty.Disease]     Alzheimer disease
-    ├── attrs:tech • 1      [Feature]
-    │   assay               cat[bionty.Experiment…  Visium Spatial Gene Express…
-    ├── attrs • 2           [Feature]
+    ├── attrs:bio (2)
+    │   developmental_sta…  bionty.DevelopmentalS…  adult stage
+    │   disease             bionty.Disease          Alzheimer disease
+    ├── attrs:tech (1)
+    │   assay               bionty.ExperimentalFa…  Visium Spatial Gene Express…
+    ├── attrs (2)
     │   bio                 dict
     │   tech                dict
-    ├── tables:table:obs …  [Feature]
+    ├── tables:table:obs …
     │   sample_region       str
-    └── tables:table:var.…  [bionty.Gene.ensembl_…
+    └── tables:table:var.…
         BRCA2               num
         BRAF                num"""
     )
