@@ -22,7 +22,7 @@ ln.connect("laminlabs/lamin-dev")
 assert ln.setup.settings.instance.slug == "laminlabs/lamin-dev"
 
 # check that the rename resolves correctly (it was renamed)
-assert ln.Artifact.using("laminlabs/lamin-dev1072025").db == "default"
+assert ln.Artifact.connect("laminlabs/lamin-dev1072025").db == "default"
 
 space_name = "Our test space for CI"
 space = ln.Space.get(name=space_name)
